@@ -463,8 +463,27 @@ function reenable_sigint(f::Function)
     res
 end
 
-function ccallable(f::Function, rt::Type, argt::Type, name::Union{AbstractString,Symbol}=string(f))
-    ccall(:jl_extern_c, Cvoid, (Any, Any, Any, Cstring), f, rt, argt, name)
+"""
+    exit_on_sigint(on::Bool)
+
+Set `exit_on_sigint` flag of the julia runtime.  If `false`, Ctrl-C
+(SIGINT) is capturable as [`InterruptException`](@ref) in `try` block.
+This is the default behavior in REPL, any code run via `-e` and `-E`
+and in Julia script run with `-i` option.
+
+If `true`, `InterruptException` is not thrown by Ctrl-C.  Running code
+upon such event requires [`atexit`](@ref).  This is the default
+behavior in Julia script run without `-i` option.
+
+!!! compat "Julia 1.5"
+    Function `exit_on_sigint` requires at least Julia 1.5.
+"""
+function exit_on_sigint(on::Bool)
+    ccall(:jl_exit_on_sigint, Cvoid, (Cint,), on)
+end
+
+function _ccallable(rt::Type, sigt::Type)
+    ccall(:jl_extern_c, Cvoid, (Any, Any), rt, sigt)
 end
 
 function expand_ccallable(rt, def)
@@ -480,17 +499,22 @@ function expand_ccallable(rt, def)
             error("@ccallable requires a return type")
         end
         if sig.head === :call
-            name = sig.args[1]
+            f = sig.args[1]
+            if isa(f,Expr) && f.head === :(::)
+                f = f.args[end]
+            else
+                f = :(typeof($f))
+            end
             at = map(sig.args[2:end]) do a
                 if isa(a,Expr) && a.head === :(::)
-                    a.args[2]
+                    a.args[end]
                 else
                     :Any
                 end
             end
             return quote
                 $(esc(def))
-                ccallable($(esc(name)), $(esc(rt)), $(Expr(:curly, :Tuple, map(esc, at)...)), $(string(name)))
+                _ccallable($(esc(rt)), $(Expr(:curly, :Tuple, esc(f), map(esc, at)...)))
             end
         end
     end
@@ -503,3 +527,176 @@ end
 macro ccallable(rt, def)
     expand_ccallable(rt, def)
 end
+
+# @ccall implementation
+"""
+    ccall_macro_parse(expression)
+
+`ccall_macro_parse` is an implementation detail of `@ccall
+
+it takes an expression like `:(printf("%d"::Cstring, value::Cuint)::Cvoid)`
+returns: a tuple of `(function_name, return_type, arg_types, args)`
+
+The above input outputs this:
+
+    (:printf, :Cvoid, [:Cstring, :Cuint], ["%d", :value])
+"""
+function ccall_macro_parse(expr::Expr)
+    # setup and check for errors
+    if !Meta.isexpr(expr, :(::))
+        throw(ArgumentError("@ccall needs a function signature with a return type"))
+    end
+    rettype = expr.args[2]
+
+    call = expr.args[1]
+    if !Meta.isexpr(call, :call)
+        throw(ArgumentError("@ccall has to take a function call"))
+    end
+
+    # get the function symbols
+    func = let f = call.args[1]
+        if Meta.isexpr(f, :.)
+            :(($(f.args[2]), $(f.args[1])))
+        elseif Meta.isexpr(f, :$)
+            f
+        elseif f isa Symbol
+            QuoteNode(f)
+        else
+            throw(ArgumentError("@ccall function name must be a symbol, a `.` node (e.g. `libc.printf`) or an interpolated function pointer (with `\$`)"))
+        end
+    end
+
+    # detect varargs
+    varargs = nothing
+    argstart = 2
+    callargs = call.args
+    if length(callargs) >= 2 && Meta.isexpr(callargs[2], :parameters)
+        argstart = 3
+        varargs = callargs[2].args
+    end
+
+    # collect args and types
+    args = []
+    types = []
+
+    function pusharg!(arg)
+        if !Meta.isexpr(arg, :(::))
+            throw(ArgumentError("args in @ccall need type annotations. '$arg' doesn't have one."))
+        end
+        push!(args, arg.args[1])
+        push!(types, arg.args[2])
+    end
+
+    for i in argstart:length(callargs)
+        pusharg!(callargs[i])
+    end
+    # add any varargs if necessary
+    nreq = 0
+    if !isnothing(varargs)
+        if length(args) == 0
+            throw(ArgumentError("C ABI prohibits vararg without one required argument"))
+        end
+        nreq = length(args)
+        for a in varargs
+            pusharg!(a)
+        end
+    end
+
+    return func, rettype, types, args, nreq
+end
+
+
+function ccall_macro_lower(convention, func, rettype, types, args, nreq)
+    lowering = []
+    realargs = []
+    gcroots = []
+
+    # if interpolation was used, ensure  variable is a function pointer at runtime.
+    if Meta.isexpr(func, :$)
+        push!(lowering, Expr(:(=), :func, esc(func.args[1])))
+        name = QuoteNode(func.args[1])
+        func = :func
+        check = quote
+            if !isa(func, Ptr{Cvoid})
+                name = $name
+                throw(ArgumentError("interpolated function `$name` was not a Ptr{Cvoid}, but $(typeof(func))"))
+            end
+        end
+        push!(lowering, check)
+    else
+        func = esc(func)
+    end
+
+    for (i, (arg, type)) in enumerate(zip(args, types))
+        sym = Symbol(string("arg", i, "root"))
+        sym2 = Symbol(string("arg", i, ))
+        earg, etype = esc(arg), esc(type)
+        push!(lowering, :($sym = Base.cconvert($etype, $earg)))
+        push!(lowering, :($sym2 = Base.unsafe_convert($etype, $sym)))
+        push!(realargs, sym2)
+        push!(gcroots, sym)
+    end
+    etypes = Expr(:call, Expr(:core, :svec), types...)
+    exp = Expr(:foreigncall,
+               func,
+               esc(rettype),
+               esc(etypes),
+               nreq,
+               QuoteNode(convention),
+               realargs..., gcroots...)
+    push!(lowering, exp)
+
+    return Expr(:block, lowering...)
+end
+
+"""
+    @ccall library.function_name(argvalue1::argtype1, ...)::returntype
+    @ccall function_name(argvalue1::argtype1, ...)::returntype
+    @ccall \$function_pointer(argvalue1::argtype1, ...)::returntype
+
+Call a function in a C-exported shared library, specified by
+`library.function_name`, where `library` is a string constant or
+literal. The library may be omitted, in which case the `function_name`
+is resolved in the current process. Alternatively, `@ccall` may
+also be used to call a function pointer `\$function_pointer`, such as
+one returned by `dlsym`.
+
+Each `argvalue` to `@ccall` is converted to the corresponding
+`argtype`, by automatic insertion of calls to `unsafe_convert(argtype,
+cconvert(argtype, argvalue))`. (See also the documentation for
+[`unsafe_convert`](@ref Base.unsafe_convert) and [`cconvert`](@ref
+Base.cconvert) for further details.) In most cases, this simply
+results in a call to `convert(argtype, argvalue)`.
+
+
+# Examples
+
+    @ccall strlen(s::Cstring)::Csize_t
+
+This calls the C standard library function:
+
+    size_t strlen(char *)
+
+with a Julia variable named `s`. See also `ccall`.
+
+Varargs are supported with the following convention:
+
+    @ccall printf("%s = %d"::Cstring ; "foo"::Cstring, foo::Cint)::Cint
+
+The semicolon is used to separate required arguments (of which there
+must be at least one) from variadic arguments.
+
+Example using an external library:
+
+    # C signature of g_uri_escape_string:
+    # char *g_uri_escape_string(const char *unescaped, const char *reserved_chars_allowed, gboolean allow_utf8);
+
+    const glib = "libglib-2.0"
+    @ccall glib.g_uri_escape_string(my_uri::Cstring, ":/"::Cstring, true::Cint)::Cstring
+
+The string literal could also be used directly before the function
+name, if desired `"libglib-2.0".g_uri_escape_string(...`
+"""
+macro ccall(expr)
+    return ccall_macro_lower(:ccall, ccall_macro_parse(expr)...)
+end
