@@ -42,12 +42,22 @@ function abstract_call_gf_by_type(@nospecialize(f), argtypes::Vector{Any}, @nosp
         splitsigs = switchtupleunion(atype)
         applicable = Any[]
         for sig_n in splitsigs
-            xapplicable = _methods_by_ftype(sig_n, max_methods, sv.params.world, min_valid, max_valid)
+            (xapplicable, min_valid[1], max_valid[1]) =
+                get!(sv.matching_methods_cache, sig_n) do
+                    ms = _methods_by_ftype(sig_n, max_methods, sv.params.world,
+                                           min_valid, max_valid)
+                    return (ms, min_valid[1], max_valid[1])
+                end
             xapplicable === false && return Any
             append!(applicable, xapplicable)
         end
     else
-        applicable = _methods_by_ftype(atype, max_methods, sv.params.world, min_valid, max_valid)
+        (applicable, min_valid[1], max_valid[1]) =
+            get!(sv.matching_methods_cache, atype) do
+                ms = _methods_by_ftype(atype, max_methods, sv.params.world,
+                                       min_valid, max_valid)
+                return (ms, min_valid[1], max_valid[1])
+            end
         if applicable === false
             # this means too many methods matched
             # (assume this will always be true, so we don't compute / update valid age in this case)
@@ -167,7 +177,7 @@ function const_prop_profitable(@nospecialize(arg))
             isconstType(b) && return true
             const_prop_profitable(b) && return true
         end
-    elseif !isa(arg, Const) || (isa(arg.val, Symbol) || isa(arg.val, Type) || (!isa(arg.val, String) && isimmutable(arg.val)))
+    elseif !isa(arg, Const) || (isa(arg.val, Symbol) || isa(arg.val, Type) || (!isa(arg.val, String) && !ismutable(arg.val)))
         # don't consider mutable values or Strings useful constants
         return true
     end
@@ -201,11 +211,15 @@ function abstract_call_method_with_const_args(@nospecialize(rettype), @nospecial
             # don't propagate constant index into indexing of non-constant array
             if arrty isa Type && arrty <: AbstractArray && !issingletontype(arrty)
                 return Any
+            elseif arrty ⊑ Array
+                return Any
             end
         elseif istopfunction(f, :iterate)
             itrty = argtypes[2]
             if itrty isa Type && !issingletontype(itrty)
                 return Any
+            elseif itrty ⊑ Array
+                return Any
             end
         end
     end
@@ -227,13 +241,13 @@ function abstract_call_method_with_const_args(@nospecialize(rettype), @nospecial
     # decide if it's likely to be worthwhile
     if !force_inference
         code = inf_for_methodinstance(mi, sv.params.world)
-        declared_inline = isdefined(method, :source) && ccall(:jl_ast_flag_inlineable, Bool, (Any,), method.source)
+        declared_inline = isdefined(method, :source) && ccall(:jl_ir_flag_inlineable, Bool, (Any,), method.source)
         cache_inlineable = declared_inline
         if isdefined(code, :inferred) && !cache_inlineable
             cache_inf = code.inferred
             if !(cache_inf === nothing)
-                cache_src_inferred = ccall(:jl_ast_flag_inferred, Bool, (Any,), cache_inf)
-                cache_src_inlineable = ccall(:jl_ast_flag_inlineable, Bool, (Any,), cache_inf)
+                cache_src_inferred = ccall(:jl_ir_flag_inferred, Bool, (Any,), cache_inf)
+                cache_src_inlineable = ccall(:jl_ir_flag_inlineable, Bool, (Any,), cache_inf)
                 cache_inlineable = cache_src_inferred && cache_src_inlineable
             end
         end
@@ -245,6 +259,7 @@ function abstract_call_method_with_const_args(@nospecialize(rettype), @nospecial
     if inf_result === nothing
         inf_result = InferenceResult(mi, argtypes)
         frame = InferenceState(inf_result, #=cache=#false, sv.params)
+        frame === nothing && return Any # this is probably a bad generated function (unsound), but just ignore it
         frame.limited = true
         frame.parent = sv
         push!(sv.params.cache, inf_result)
@@ -450,7 +465,7 @@ function precise_container_type(@nospecialize(itft), @nospecialize(typ), vtypes:
     tti0 = widenconst(typ)
     tti = unwrap_unionall(tti0)
     if isa(tti, DataType) && tti.name === NamedTuple_typename
-        # A NamedTuple iteration is the the same as the iteration of its Tuple parameter:
+        # A NamedTuple iteration is the same as the iteration of its Tuple parameter:
         # compute a new `tti == unwrap_unionall(tti0)` based on that Tuple type
         tti = tti.parameters[2]
         while isa(tti, TypeVar)
@@ -514,6 +529,7 @@ function abstract_iteration(@nospecialize(itft), @nospecialize(itertype), vtypes
     else
         return Any[Vararg{Any}]
     end
+    @assert !isvarargtype(itertype)
     stateordonet = abstract_call_known(iteratef, nothing, Any[itft, itertype], vtypes, sv)
     # Return Bottom if this is not an iterator.
     # WARNING: Changes to the iteration protocol must be reflected here,
@@ -576,7 +592,22 @@ function abstract_apply(@nospecialize(itft), @nospecialize(aft), aargtypes::Vect
     for i = 1:nargs
         ctypes´ = []
         for ti in (splitunions ? uniontypes(aargtypes[i]) : Any[aargtypes[i]])
-            cti = precise_container_type(itft, ti, vtypes, sv)
+            if !isvarargtype(ti)
+                cti = precise_container_type(itft, ti, vtypes, sv)
+            else
+                cti = precise_container_type(itft, unwrapva(ti), vtypes, sv)
+                # We can't represent a repeating sequence of the same types,
+                # so tmerge everything together to get one type that represents
+                # everything.
+                argt = cti[end]
+                if isvarargtype(argt)
+                    argt = unwrapva(argt)
+                end
+                for i in 1:(length(cti)-1)
+                    argt = tmerge(argt, cti[i])
+                end
+                cti = Any[Vararg{argt}]
+            end
             if _any(t -> t === Bottom, cti)
                 continue
             end
@@ -1011,6 +1042,17 @@ function abstract_eval(@nospecialize(e), vtypes::VarTable, sv::InferenceState)
         end
     elseif e.head === :splatnew
         t = instanceof_tfunc(abstract_eval(e.args[1], vtypes, sv))[1]
+        if length(e.args) == 2 && isconcretetype(t) && !t.mutable
+            at = abstract_eval(e.args[2], vtypes, sv)
+            n = fieldcount(t)
+            if isa(at, Const) && isa(at.val, Tuple) && n == length(at.val) &&
+                    _all(i->at.val[i] isa fieldtype(t, i), 1:n)
+                t = Const(ccall(:jl_new_structt, Any, (Any, Any), t, at.val))
+            elseif isa(at, PartialStruct) && at ⊑ Tuple && n == length(at.fields) &&
+                    _all(i->at.fields[i] ⊑ fieldtype(t, i), 1:n)
+                t = PartialStruct(t, at.fields)
+            end
+        end
     elseif e.head === :&
         abstract_eval(e.args[1], vtypes, sv)
         t = Any
@@ -1222,7 +1264,8 @@ function typeinf_local(frame::InferenceState)
                     if isa(fname, Slot)
                         changes = StateUpdate(fname, VarState(Any, false), changes)
                     end
-                elseif hd === :inbounds || hd === :meta || hd === :loopinfo
+                elseif hd === :inbounds || hd === :meta || hd === :loopinfo || hd == :code_coverage_effect
+                    # these do not generate code
                 else
                     t = abstract_eval(stmt, changes, frame)
                     t === Bottom && break
