@@ -9,6 +9,7 @@ struct JLOptions
     commands::Ptr{Ptr{UInt8}} # (e)eval, (E)print, (L)load
     image_file::Ptr{UInt8}
     cpu_target::Ptr{UInt8}
+    nthreads::Int32
     nprocs::Int32
     machine_file::Ptr{UInt8}
     project::Ptr{UInt8}
@@ -36,11 +37,13 @@ struct JLOptions
     bindto::Ptr{UInt8}
     outputbc::Ptr{UInt8}
     outputunoptbc::Ptr{UInt8}
-    outputjitbc::Ptr{UInt8}
     outputo::Ptr{UInt8}
+    outputasm::Ptr{UInt8}
     outputji::Ptr{UInt8}
     output_code_coverage::Ptr{UInt8}
     incremental::Int8
+    image_file_specified::Int8
+    warn_scope::Int8
 end
 
 # This runs early in the sysimage != is not defined yet
