@@ -340,6 +340,23 @@ matching sequence is found, like the return value of [`findnext`](@ref).
 
 If `overlap=true`, the matching sequences are allowed to overlap indices in the
 original string, otherwise they must be from disjoint character ranges.
+
+# Examples
+```jldoctest
+julia> findall("a", "apple")
+1-element Array{UnitRange{Int64},1}:
+ 1:1
+
+julia> findall("nana", "banana")
+1-element Array{UnitRange{Int64},1}:
+ 3:6
+
+julia> findall("a", "banana")
+3-element Array{UnitRange{Int64},1}:
+ 2:2
+ 4:4
+ 6:6
+```
 """
 function findall(t::Union{AbstractString,Regex}, s::AbstractString; overlap::Bool=false)
     found = UnitRange{Int}[]
