@@ -37,60 +37,53 @@
 //===----------------------------------------------------------------------===//
 
 
-typedef bool AbiState;
-AbiState default_abi_state = 0;
+struct ABI_x86Layout : AbiLayout {
 
-inline bool is_complex64(jl_value_t *ty)
+STATIC_INLINE bool is_complex_type(jl_datatype_t *dt)
 {
-    return jl_complex_type != NULL && jl_is_datatype(ty) &&
-        ((jl_datatype_t*)ty)->name == jl_complex_type->name &&
-        jl_tparam0(ty) == (jl_value_t*)jl_float32_type;
+    static jl_sym_t *Complex_sym = NULL;
+    if (Complex_sym == NULL)
+        Complex_sym = jl_symbol("Complex");
+    return jl_is_datatype(dt) && dt->name->name == Complex_sym && dt->name->module == jl_base_module;
 }
 
-inline bool is_complex128(jl_value_t *ty)
+inline bool is_complex64(jl_datatype_t *dt) const
 {
-    return jl_complex_type != NULL && jl_is_datatype(ty) &&
-        ((jl_datatype_t*)ty)->name == jl_complex_type->name &&
-        jl_tparam0(ty) == (jl_value_t*)jl_float64_type;
+    return is_complex_type(dt) && jl_tparam0(dt) == (jl_value_t*)jl_float32_type;
 }
 
-bool use_sret(AbiState *state, jl_value_t *ty)
+inline bool is_complex128(jl_datatype_t *dt) const
 {
-    // Assume (jl_is_datatype(ty) && !jl_is_abstracttype(ty) &&
-    //         !jl_is_array_type(ty))
-    if (jl_is_cpointer_type(ty))
-        return false;
-    size_t size = jl_datatype_size(ty);
+    return is_complex_type(dt) && jl_tparam0(dt) == (jl_value_t*)jl_float64_type;
+}
+
+bool use_sret(jl_datatype_t *dt) override
+{
+    size_t size = jl_datatype_size(dt);
     if (size == 0)
         return false;
-    if (is_complex64(ty) || (jl_is_bitstype(ty) && size <= 8))
+    if (is_complex64(dt) || (jl_is_primitivetype(dt) && size <= 8))
         return false;
     return true;
 }
 
-void needPassByRef(AbiState *state, jl_value_t *ty, bool *byRef, bool *inReg, bool *byRefAttr)
+bool needPassByRef(jl_datatype_t *dt, AttrBuilder &ab) override
 {
-    if (!jl_is_datatype(ty) || jl_is_abstracttype(ty) || jl_is_cpointer_type(ty) || jl_is_array_type(ty))
-        return;
-    size_t size = jl_datatype_size(ty);
-    if (is_complex64(ty) || is_complex128(ty) || (jl_is_bitstype(ty) && size <= 8))
-        return;
-    *byRefAttr = *byRef = true;
+    size_t size = jl_datatype_size(dt);
+    if (is_complex64(dt) || is_complex128(dt) || (jl_is_primitivetype(dt) && size <= 8))
+        return false;
+    ab.addAttribute(Attribute::ByVal);
+    return true;
 }
 
-Type *preferred_llvm_type(jl_value_t *ty, bool isret)
+Type *preferred_llvm_type(jl_datatype_t *dt, bool isret) const override
 {
     if (!isret)
         return NULL;
-    if (!jl_is_datatype(ty) || jl_is_abstracttype(ty) || jl_is_cpointer_type(ty) || jl_is_array_type(ty))
-        return NULL;
     // special case Complex{Float32} as a return type
-    if (is_complex64(ty))
+    if (is_complex64(dt))
         return T_int64;
     return NULL;
 }
 
-bool need_private_copy(jl_value_t *ty, bool byRef)
-{
-    return false;
-}
+};
