@@ -33,9 +33,14 @@ DECLARE_BUILTIN(arrayset);   DECLARE_BUILTIN(arraysize);
 DECLARE_BUILTIN(apply_type); DECLARE_BUILTIN(applicable);
 DECLARE_BUILTIN(invoke);     DECLARE_BUILTIN(_expr);
 DECLARE_BUILTIN(typeassert); DECLARE_BUILTIN(ifelse);
-DECLARE_BUILTIN(_typevar);
+DECLARE_BUILTIN(_typevar);   DECLARE_BUILTIN(_typebody);
 
 JL_CALLABLE(jl_f_invoke_kwsorter);
+JL_CALLABLE(jl_f__structtype);
+JL_CALLABLE(jl_f__abstracttype);
+JL_CALLABLE(jl_f__primitivetype);
+JL_CALLABLE(jl_f__setsuper);
+JL_CALLABLE(jl_f__equiv_typedef);
 
 #ifdef __cplusplus
 }
