@@ -35,6 +35,7 @@ jl_options_t jl_options = { 0,    // quiet
                             NULL, // cmds
                             NULL, // image_file (will be filled in below)
                             NULL, // cpu_target ("native", "core2", etc...)
+                            0,    // nthreads
                             0,    // nprocs
                             NULL, // machine_file
                             NULL, // project
@@ -52,7 +53,7 @@ jl_options_t jl_options = { 0,    // quiet
                             1,    // debug_level [release build]
 #endif
                             JL_OPTIONS_CHECK_BOUNDS_DEFAULT, // check_bounds
-                            JL_OPTIONS_DEPWARN_ON,    // deprecation warning
+                            JL_OPTIONS_DEPWARN_OFF,    // deprecation warning
                             0,    // method overwrite warning
                             1,    // can_inline
                             JL_OPTIONS_POLLY_ON, // polly
@@ -66,18 +67,20 @@ jl_options_t jl_options = { 0,    // quiet
                             NULL, // bind-to
                             NULL, // output-bc
                             NULL, // output-unopt-bc
-                            NULL, // output-jit-bc
                             NULL, // output-o
+                            NULL, // output-asm
                             NULL, // output-ji
                             NULL,    // output-code_coverage
                             0, // incremental
-                            0 // image_file_specified
+                            0, // image_file_specified
+                            JL_OPTIONS_WARN_SCOPE_ON  // ambiguous scope warning
 };
 
 static const char usage[] = "julia [switches] -- [programfile] [args...]\n";
 static const char opts[]  =
     " -v, --version             Display version information\n"
-    " -h, --help                Print this message (--help-hidden for more)\n\n"
+    " -h, --help                Print this message (--help-hidden for more)\n"
+    " --help-hidden             Uncommon options not shown by `-h`\n\n"
 
     // startup options
     " --project[={<dir>|@.}]    Set <dir> as the home project/environment\n"
@@ -96,6 +99,8 @@ static const char opts[]  =
     " -L, --load <file>         Load <file> immediately on all processors\n\n"
 
     // parallel options
+    " -t, --threads {N|auto}    Enable N threads; \"auto\" currently sets N to the number of local\n"
+    "                           CPU threads but this might change in the future\n"
     " -p, --procs {N|auto}      Integer value N launches N additional local worker processes\n"
     "                           \"auto\" launches as many workers as the number of local CPU threads (logical cores)\n"
     " --machine-file <file>     Run processes on hosts listed in <file>\n\n"
@@ -109,7 +114,8 @@ static const char opts[]  =
 
     // error and warning options
     " --depwarn={yes|no|error}  Enable or disable syntax and method deprecation warnings (\"error\" turns warnings into errors)\n"
-    " --warn-overwrite={yes|no} Enable or disable method overwrite warnings\n\n"
+    " --warn-overwrite={yes|no} Enable or disable method overwrite warnings\n"
+    " --warn-scope={yes|no}     Enable or disable warning for ambiguous top-level scope\n\n"
 
     // code generation options
     " -C, --cpu-target <target> Limit usage of CPU features up to <target>; set to \"help\" to see the available options\n"
@@ -134,7 +140,11 @@ static const char opts[]  =
     "                           Append coverage information to the LCOV tracefile (filename supports format tokens).\n"
 // TODO: These TOKENS are defined in `runtime_ccall.cpp`. A more verbose `--help` should include that list here.
     " --track-allocation={none|user|all}, --track-allocation\n"
-    "                           Count bytes allocated by each source line (omitting setting is equivalent to \"user\")\n\n"
+    "                           Count bytes allocated by each source line (omitting setting is equivalent to \"user\")\n"
+    " --bug-report=KIND         Launch a bug report session. It can be used to start a REPL, run a script, or evaluate\n"
+    "                           expressions. It first tries to use BugReporting.jl installed in current environment and\n"
+    "                           fallbacks to the latest compatible BugReporting.jl if not. For more information, see\n"
+    "                           --bug-report=help.\n\n"
 ;
 
 static const char opts_hidden[]  =
@@ -149,6 +159,7 @@ static const char opts_hidden[]  =
     " --output-unopt-bc name    Generate unoptimized LLVM bitcode (.bc)\n"
     " --output-jit-bc name      Dump all IR generated by the frontend (not including system image)\n"
     " --output-bc name          Generate LLVM bitcode (.bc)\n"
+    " --output-asm name         Generate an assembly file (.s)\n"
     " --output-incremental=no   Generate an incremental output file (rather than complete)\n"
     " --trace-compile={stdout,stderr}\n"
     "                           Print precompile statements for methods compiled during execution.\n\n"
@@ -164,11 +175,11 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
            opt_code_coverage,
            opt_track_allocation,
            opt_check_bounds,
-           opt_output_jit_bc,
            opt_output_unopt_bc,
            opt_output_bc,
            opt_depwarn,
            opt_warn_overwrite,
+           opt_warn_scope,
            opt_inline,
            opt_polly,
            opt_trace_compile,
@@ -177,6 +188,7 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
            opt_bind_to,
            opt_handle_signals,
            opt_output_o,
+           opt_output_asm,
            opt_output_ji,
            opt_use_precompiled,
            opt_use_compilecache,
@@ -187,8 +199,9 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
            opt_compiled_modules,
            opt_machine_file,
            opt_project,
+           opt_bug_report
     };
-    static const char* const shortopts = "+vhqH:e:E:L:J:C:ip:O:g:";
+    static const char* const shortopts = "+vhqH:e:E:L:J:C:it:p:O:g:";
     static const struct option longopts[] = {
         // exposed command line options
         // NOTE: This set of required arguments need to be kept in sync
@@ -202,11 +215,13 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
         { "eval",            required_argument, 0, 'e' },
         { "print",           required_argument, 0, 'E' },
         { "load",            required_argument, 0, 'L' },
+        { "bug-report",      required_argument, 0, opt_bug_report },
         { "sysimage",        required_argument, 0, 'J' },
         { "sysimage-native-code", required_argument, 0, opt_sysimage_native_code },
         { "compiled-modules",    required_argument, 0, opt_compiled_modules },
         { "cpu-target",      required_argument, 0, 'C' },
         { "procs",           required_argument, 0, 'p' },
+        { "threads",         required_argument, 0, 't' },
         { "machine-file",    required_argument, 0, opt_machine_file },
         { "project",         optional_argument, 0, opt_project },
         { "color",           required_argument, 0, opt_color },
@@ -219,12 +234,13 @@ JL_DLLEXPORT void jl_parse_opts(int *argcp, char ***argvp)
         { "check-bounds",    required_argument, 0, opt_check_bounds },
         { "output-bc",       required_argument, 0, opt_output_bc },
         { "output-unopt-bc", required_argument, 0, opt_output_unopt_bc },
-        { "output-jit-bc",   required_argument, 0, opt_output_jit_bc },
         { "output-o",        required_argument, 0, opt_output_o },
+        { "output-asm",      required_argument, 0, opt_output_asm },
         { "output-ji",       required_argument, 0, opt_output_ji },
         { "output-incremental",required_argument, 0, opt_incremental },
         { "depwarn",         required_argument, 0, opt_depwarn },
         { "warn-overwrite",  required_argument, 0, opt_warn_overwrite },
+        { "warn-scope",      required_argument, 0, opt_warn_scope },
         { "inline",          required_argument, 0, opt_inline },
         { "polly",           required_argument, 0, opt_polly },
         { "trace-compile",   required_argument, 0, opt_trace_compile },
@@ -283,7 +299,7 @@ restart_switch:
                             c = o->val;
                             goto restart_switch;
                         }
-                        else if (strchr(shortopts, o->val)) {
+                        else if (o->val <= 0xff && strchr(shortopts, o->val)) {
                             jl_errorf("option `-%c/--%s` is missing an argument", o->val, o->name);
                         }
                         else {
@@ -330,11 +346,12 @@ restart_switch:
         case 'e': // eval
         case 'E': // print
         case 'L': // load
+        case opt_bug_report: // bug
         {
             size_t sz = strlen(optarg) + 1;
             char *arg = (char*)malloc_s(sz + 1);
             const char **newcmds;
-            arg[0] = c;
+            arg[0] = c == opt_bug_report ? 'B' : c;
             memcpy(arg + 1, optarg, sz);
             newcmds = (const char**)realloc_s(cmds, (ncmds + 2) * sizeof(char*));
             cmds = newcmds;
@@ -386,6 +403,18 @@ restart_switch:
             if (!jl_options.cpu_target)
                 jl_error("julia: failed to allocate memory");
             break;
+        case 't': // threads
+            errno = 0;
+            if (!strcmp(optarg,"auto")) {
+                jl_options.nthreads = -1;
+            }
+            else {
+                long nthreads = strtol(optarg, &endptr, 10);
+                if (errno != 0 || optarg == endptr || *endptr != 0 || nthreads < 1 || nthreads >= INT_MAX)
+                    jl_errorf("julia: -t,--threads=<n> must be an integer >= 1");
+                jl_options.nthreads = (int)nthreads;
+            }
+            break;
         case 'p': // procs
             errno = 0;
             if (!strcmp(optarg,"auto")) {
@@ -515,9 +544,6 @@ restart_switch:
             jl_options.outputbc = optarg;
             if (!jl_options.image_file_specified) jl_options.image_file = NULL;
             break;
-        case opt_output_jit_bc:
-            jl_options.outputjitbc = optarg;
-            break;
         case opt_output_unopt_bc:
             jl_options.outputunoptbc = optarg;
             if (!jl_options.image_file_specified) jl_options.image_file = NULL;
@@ -526,6 +552,10 @@ restart_switch:
             jl_options.outputo = optarg;
             if (!jl_options.image_file_specified) jl_options.image_file = NULL;
             break;
+        case opt_output_asm:
+            jl_options.outputasm = optarg;
+            if (!jl_options.image_file_specified) jl_options.image_file = NULL;
+            break;
         case opt_output_ji:
             jl_options.outputji = optarg;
             if (!jl_options.image_file_specified) jl_options.image_file = NULL;
@@ -554,7 +584,15 @@ restart_switch:
             else if (!strcmp(optarg,"no"))
                 jl_options.warn_overwrite = JL_OPTIONS_WARN_OVERWRITE_OFF;
             else
-                jl_errorf("julia: invalid argument to --warn-overwrite={yes|no|} (%s)", optarg);
+                jl_errorf("julia: invalid argument to --warn-overwrite={yes|no} (%s)", optarg);
+            break;
+        case opt_warn_scope:
+            if (!strcmp(optarg,"yes"))
+                jl_options.warn_scope = JL_OPTIONS_WARN_SCOPE_ON;
+            else if (!strcmp(optarg,"no"))
+                jl_options.warn_scope = JL_OPTIONS_WARN_SCOPE_OFF;
+            else
+                jl_errorf("julia: invalid argument to --warn-scope={yes|no} (%s)", optarg);
             break;
         case opt_inline:
             if (!strcmp(optarg,"yes"))
