@@ -7,7 +7,6 @@
 // LLVM pass to clone function for different archs
 
 #include "llvm-version.h"
-#include "support/dtypes.h"
 
 #include <llvm-c/Core.h>
 #include <llvm-c/Types.h>
@@ -29,6 +28,7 @@
 #include "julia.h"
 #include "julia_internal.h"
 #include "processor.h"
+#include "support/dtypes.h"
 
 #include <map>
 #include <memory>
@@ -625,7 +625,7 @@ void CloneCtx::add_features(Function *F, StringRef name, StringRef features, uin
 {
     auto attr = F->getFnAttribute("target-features");
     if (attr.isStringAttribute()) {
-        std::string new_features = attr.getValueAsString();
+        std::string new_features(attr.getValueAsString());
         new_features += ",";
         new_features += features;
         F->addFnAttr("target-features", new_features);
