@@ -3,9 +3,6 @@
 #include "llvm-version.h"
 
 #define DEBUG_TYPE "lower_simd_loop"
-#if JL_LLVM_VERSION < 70000
-#define LLVM_DEBUG DEBUG
-#endif
 
 // This file defines a LLVM pass that:
 // 1. Set's loop information in form of metadata
