@@ -17,16 +17,36 @@ extern "C" {
 
 // thread sleep state
 
-static int16_t sleep_check_state; // status of the multi-queue. possible values:
-
-// no thread should be sleeping--there might be work in the multi-queue.
+// thread should not be sleeping--it might need to do work.
 static const int16_t not_sleeping = 0;
 
-// it is acceptable for a thread to be sleeping if its sticky queue is empty.
-// sleep_check_state == sleeping + 1 + tid means thread tid is checking the multi-queue
-// to see if it is safe to transition to sleeping.
+// it is acceptable for the thread to be sleeping.
 static const int16_t sleeping = 1;
 
+// invariant: No thread is ever asleep unless sleep_check_state is sleeping (or we have a wakeup signal pending).
+// invariant: Any particular thread is not asleep unless that thread's sleep_check_state is sleeping.
+// invariant: The transition of a thread state to sleeping must be followed by a check that there wasn't work pending for it.
+// information: Observing thread not-sleeping is sufficient to ensure the target thread will subsequently inspect its local queue.
+// information: Observing thread is-sleeping says it may be necessary to notify it at least once to wakeup. It may already be awake however for a variety of reasons.
+
+JULIA_DEBUG_SLEEPWAKE(
+uint64_t wakeup_enter;
+uint64_t wakeup_leave;
+uint64_t io_wakeup_enter;
+uint64_t io_wakeup_leave;
+);
+
+
+JL_DLLEXPORT int jl_set_task_tid(jl_task_t *task, int tid) JL_NOTSAFEPOINT
+{
+    // Try to acquire the lock on this task.
+    int16_t was = task->tid;
+    if (was == tid)
+        return 1;
+    if (was == -1)
+        return jl_atomic_bool_compare_exchange(&task->tid, -1, tid);
+    return 0;
+}
 
 // GC functions used
 extern int jl_gc_mark_queue_obj_explicit(jl_gc_mark_cache_t *gc_cache,
@@ -160,11 +180,9 @@ static inline jl_task_t *multiq_deletemin(void)
         return NULL;
 
     task = heaps[rn1].tasks[0];
-    if (jl_atomic_load_acquire(&task->tid) != ptls->tid) {
-        if (jl_atomic_compare_exchange(&task->tid, -1, ptls->tid) != -1) {
-            jl_mutex_unlock_nogc(&heaps[rn1].lock);
-            goto retry;
-        }
+    if (!jl_set_task_tid(task, ptls->tid)) {
+        jl_mutex_unlock_nogc(&heaps[rn1].lock);
+        goto retry;
     }
     heaps[rn1].tasks[0] = heaps[rn1].tasks[--heaps[rn1].ntasks];
     heaps[rn1].tasks[heaps[rn1].ntasks] = NULL;
@@ -200,43 +218,6 @@ static int multiq_check_empty(void)
 }
 
 
-static int sleep_check_now(int16_t tid)
-{
-    while (1) {
-        int16_t state = jl_atomic_load(&sleep_check_state);
-        if (state > sleeping) {
-            // if some thread is already checking, the decision of that thread
-            // is correct for us also
-            do {
-                state = jl_atomic_load(&sleep_check_state);
-            } while (state > sleeping);
-            if (state == not_sleeping)
-                return 0;
-        }
-        else if (state == not_sleeping) {
-            int16_t checking_for_sleeping = sleeping + 1 + tid;
-            // transition from sleeping ==> checking
-            if (jl_atomic_bool_compare_exchange(&sleep_check_state, not_sleeping,
-                                                checking_for_sleeping)) {
-                if (multiq_check_empty()) {
-                    // transition from checking ==> sleeping
-                    if (jl_atomic_bool_compare_exchange(&sleep_check_state, checking_for_sleeping,
-                                                        sleeping))
-                        return 1;
-                }
-                else {
-                    // transition from checking ==> not_sleeping
-                    jl_atomic_store(&sleep_check_state, not_sleeping);
-                    return 0;
-                }
-            }
-            continue;
-        }
-        assert(state == sleeping);
-        return 1;
-    }
-}
-
 
 // parallel task runtime
 // ---
@@ -250,7 +231,6 @@ void jl_init_threadinginfra(void)
     jl_ptls_t ptls = jl_get_ptls_states();
     uv_mutex_init(&ptls->sleep_lock);
     uv_cond_init(&ptls->wake_signal);
-    sleep_check_state = not_sleeping;
 }
 
 
@@ -295,9 +275,45 @@ JL_DLLEXPORT int jl_enqueue_task(jl_task_t *task)
 }
 
 
+static int running_under_rr(void)
+{
+#ifdef _OS_LINUX_
+#define RR_CALL_BASE 1000
+#define SYS_rrcall_check_presence (RR_CALL_BASE + 8)
+    static int checked_running_under_rr = 0;
+    static int is_running_under_rr = 0;
+    if (!checked_running_under_rr) {
+        int ret = syscall(SYS_rrcall_check_presence, 0, 0, 0, 0, 0, 0);
+        if (ret == -1) {
+            // Should always be ENOSYS, but who knows what people do for
+            // unknown syscalls with their seccomp filters, so just say
+            // that we don't have rr.
+            is_running_under_rr = 0;
+        }
+        else {
+            is_running_under_rr = 1;
+        }
+        checked_running_under_rr = 1;
+    }
+    return is_running_under_rr;
+#else
+    return 0;
+#endif
+}
+
+
 //  sleep_check_after_threshold() -- if sleep_threshold ns have passed, return 1
 static int sleep_check_after_threshold(uint64_t *start_cycles)
 {
+    JULIA_DEBUG_SLEEPWAKE( return 1 ); // hammer on the sleep/wake logic much harder
+    /**
+     * This wait loop is a bit of a worst case for rr - it needs timer access,
+     * which are slow and it busy loops in user space, which prevents the
+     * scheduling logic from switching to other threads. Just don't bother
+     * trying to wait here
+     */
+    if (running_under_rr())
+        return 1;
     if (!(*start_cycles)) {
         *start_cycles = jl_hrtime();
         return 0;
@@ -314,8 +330,7 @@ static int sleep_check_after_threshold(uint64_t *start_cycles)
 static void wake_thread(int16_t tid)
 {
     jl_ptls_t other = jl_all_tls_states[tid];
-    int16_t state = jl_atomic_exchange(&other->sleep_check_state, not_sleeping);
-    if (state == sleeping) {
+    if (jl_atomic_bool_compare_exchange(&other->sleep_check_state, sleeping, not_sleeping)) {
         uv_mutex_lock(&other->sleep_lock);
         uv_cond_signal(&other->wake_signal);
         uv_mutex_unlock(&other->sleep_lock);
@@ -323,16 +338,25 @@ static void wake_thread(int16_t tid)
 }
 
 
+static void wake_libuv(void)
+{
+    JULIA_DEBUG_SLEEPWAKE( io_wakeup_enter = cycleclock() );
+    jl_wake_libuv();
+    JULIA_DEBUG_SLEEPWAKE( io_wakeup_leave = cycleclock() );
+}
+
 /* ensure thread tid is awake if necessary */
 JL_DLLEXPORT void jl_wakeup_thread(int16_t tid)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
+    int16_t uvlock = jl_atomic_load(&jl_uv_mutex.owner);
     int16_t self = ptls->tid;
     unsigned long system_self = jl_all_tls_states[self]->system_id;
-    int16_t uvlock = jl_atomic_load_acquire(&jl_uv_mutex.owner);
+    JULIA_DEBUG_SLEEPWAKE( wakeup_enter = cycleclock() );
     if (tid == self || tid == -1) {
         // we're already awake, but make sure we'll exit uv_run
-        jl_atomic_store(&ptls->sleep_check_state, not_sleeping);
+        if (jl_atomic_load_relaxed(&ptls->sleep_check_state) == sleeping)
+            jl_atomic_store(&ptls->sleep_check_state, not_sleeping);
         if (uvlock == system_self)
             uv_stop(jl_global_event_loop());
     }
@@ -340,38 +364,27 @@ JL_DLLEXPORT void jl_wakeup_thread(int16_t tid)
         // something added to the sticky-queue: notify that thread
         wake_thread(tid);
         // check if we need to notify uv_run too
-        if (uvlock != system_self)
-            jl_wake_libuv();
+        unsigned long system_tid = jl_all_tls_states[tid]->system_id;
+        if (uvlock != system_self && jl_atomic_load(&jl_uv_mutex.owner) == system_tid)
+            wake_libuv();
     }
+    // check if the other threads might be sleeping
     if (tid == -1) {
-        // check if the other threads might be sleeping
-        if (jl_atomic_load_acquire(&sleep_check_state) != not_sleeping) {
-            // something added to the multi-queue: notify all threads
-            // in the future, we might want to instead wake some fraction of threads,
-            // and let each of those wake additional threads if they find work
-            int16_t state = jl_atomic_exchange(&sleep_check_state, not_sleeping);
-            if (state == sleeping) {
-                for (tid = 0; tid < jl_n_threads; tid++)
-                    if (tid != self)
-                        wake_thread(tid);
-                // check if we need to notify uv_run too
-                if (uvlock != system_self)
-                    jl_wake_libuv();
-            }
+        // something added to the multi-queue: notify all threads
+        // in the future, we might want to instead wake some fraction of threads,
+        // and let each of those wake additional threads if they find work
+        for (tid = 0; tid < jl_n_threads; tid++) {
+            if (tid != self)
+                wake_thread(tid);
         }
+        // check if we need to notify uv_run too
+        if (uvlock != system_self && jl_atomic_load(&jl_uv_mutex.owner) != 0)
+            wake_libuv();
     }
+    JULIA_DEBUG_SLEEPWAKE( wakeup_leave = cycleclock() );
 }
 
 
-JL_DLLEXPORT void jl_set_task_tid(jl_task_t *task, int tid) JL_NOTSAFEPOINT
-{
-    // Try to acquire the lock on this task.
-    // If this fails, we'll check for that error later (in jl_switchto).
-    if (jl_atomic_load_acquire(&task->tid) != tid) {
-        jl_atomic_compare_exchange(&task->tid, -1, tid);
-    }
-}
-
 // get the next runnable task from the multiq
 static jl_task_t *get_next_task(jl_value_t *trypoptask, jl_value_t *q)
 {
@@ -389,7 +402,10 @@ static jl_task_t *get_next_task(jl_value_t *trypoptask, jl_value_t *q)
 
 static int may_sleep(jl_ptls_t ptls)
 {
-    return jl_atomic_load(&sleep_check_state) == sleeping && jl_atomic_load(&ptls->sleep_check_state) == sleeping;
+    // sleep_check_state is only transitioned from not_sleeping to sleeping
+    // by the thread itself. As a result, if this returns false, it will
+    // continue returning false. If it returns true, there are no guarantees.
+    return jl_atomic_load_relaxed(&ptls->sleep_check_state) == sleeping;
 }
 
 extern volatile unsigned _threadedregion;
@@ -414,12 +430,19 @@ JL_DLLEXPORT jl_task_t *jl_task_get_next(jl_value_t *trypoptask, jl_value_t *q)
 
         jl_cpu_pause();
         if (sleep_check_after_threshold(&start_cycles) || (!_threadedregion && ptls->tid == 0)) {
-            if (!sleep_check_now(ptls->tid))
-                continue;
             jl_atomic_store(&ptls->sleep_check_state, sleeping); // acquire sleep-check lock
+            if (!multiq_check_empty()) {
+                if (ptls->sleep_check_state != not_sleeping)
+                    jl_atomic_store(&ptls->sleep_check_state, not_sleeping); // let other threads know they don't need to wake us
+                continue;
+            }
             task = get_next_task(trypoptask, q);
-            if (task)
+            if (task) {
+                if (ptls->sleep_check_state != not_sleeping)
+                    jl_atomic_store(&ptls->sleep_check_state, not_sleeping); // let other threads know they don't need to wake us
                 return task;
+            }
+
             // one thread should win this race and watch the event loop
             // inside a threaded region, any thread can listen for IO messages,
             // although none are allowed to create new ones
@@ -431,7 +454,7 @@ JL_DLLEXPORT jl_task_t *jl_task_get_next(jl_value_t *trypoptask, jl_value_t *q)
             }
             else if (ptls->tid == 0) {
                 uvlock = 1;
-                JL_UV_LOCK();
+                JL_UV_LOCK(); // jl_mutex_lock(&jl_uv_mutex);
             }
             if (uvlock) {
                 int active = 1;
@@ -446,11 +469,14 @@ JL_DLLEXPORT jl_task_t *jl_task_get_next(jl_value_t *trypoptask, jl_value_t *q)
                     jl_gc_safepoint();
                     if (may_sleep(ptls)) {
                         loop->stop_flag = 0;
+                        JULIA_DEBUG_SLEEPWAKE( ptls->uv_run_enter = cycleclock() );
                         active = uv_run(loop, UV_RUN_ONCE);
+                        JULIA_DEBUG_SLEEPWAKE( ptls->uv_run_leave = cycleclock() );
                     }
                     JL_UV_UNLOCK();
                     // optimization: check again first if we may have work to do
                     if (!may_sleep(ptls)) {
+                        assert(ptls->sleep_check_state == not_sleeping);
                         start_cycles = 0;
                         continue;
                     }
@@ -468,30 +494,30 @@ JL_DLLEXPORT jl_task_t *jl_task_get_next(jl_value_t *trypoptask, jl_value_t *q)
                 if (!_threadedregion && active && ptls->tid == 0) {
                     // thread 0 is the only thread permitted to run the event loop
                     // so it needs to stay alive
+                    if (jl_atomic_load_relaxed(&ptls->sleep_check_state) != not_sleeping)
+                        jl_atomic_store(&ptls->sleep_check_state, not_sleeping); // let other threads know they don't need to wake us
                     start_cycles = 0;
                     continue;
                 }
             }
 
             // the other threads will just wait for on signal to resume
+            JULIA_DEBUG_SLEEPWAKE( ptls->sleep_enter = cycleclock() );
             int8_t gc_state = jl_gc_safe_enter(ptls);
             uv_mutex_lock(&ptls->sleep_lock);
             while (may_sleep(ptls)) {
                 uv_cond_wait(&ptls->wake_signal, &ptls->sleep_lock);
+                // TODO: help with gc work here, if applicable
             }
+            assert(ptls->sleep_check_state == not_sleeping);
             uv_mutex_unlock(&ptls->sleep_lock);
+            JULIA_DEBUG_SLEEPWAKE( ptls->sleep_leave = cycleclock() );
             jl_gc_safe_leave(ptls, gc_state); // contains jl_gc_safepoint
             start_cycles = 0;
         }
         else {
-#ifndef JL_HAVE_ASYNCIFY
             // maybe check the kernel for new messages too
-            if (jl_atomic_load(&jl_uv_n_waiters) == 0)
-                jl_process_events();
-#else
-            // Yield back to browser event loop
-            return ptls->root_task;
-#endif
+            jl_process_events();
         }
     }
 }
