@@ -4,6 +4,7 @@
 #include <map>
 #include <string>
 #include <cstdio>
+#include <llvm/ADT/StringMap.h>
 #include <llvm/Support/Host.h>
 #include <llvm/Support/raw_ostream.h>
 
@@ -136,18 +137,21 @@ std::string jl_format_filename(StringRef output_pattern)
     for (auto c : output_pattern) {
         if (special) {
             if (!got_pwd && (c == 'i' || c == 'd' || c == 'u')) {
-                uv_os_get_passwd(&pwd);
-                got_pwd = true;
+                int r = uv_os_get_passwd(&pwd);
+                if (r == 0)
+                    got_pwd = true;
             }
             switch (c) {
             case 'p':
                 outfile << jl_getpid();
                 break;
             case 'd':
-                outfile << pwd.homedir;
+                if (got_pwd)
+                    outfile << pwd.homedir;
                 break;
             case 'i':
-                outfile << pwd.uid;
+                if (got_pwd)
+                    outfile << pwd.uid;
                 break;
             case 'l':
             case 'L':
@@ -163,7 +167,8 @@ std::string jl_format_filename(StringRef output_pattern)
 #endif
                 break;
             case 'u':
-                outfile << pwd.username;
+                if (got_pwd)
+                    outfile << pwd.username;
                 break;
             default:
                 outfile << c;
