@@ -6,7 +6,7 @@
 
 // Copied from MINGW_FLOAT_H which may not be found due to a collision with the builtin gcc float.h
 // eventually we can probably integrate this into OpenLibm.
-#if defined(_COMPILER_MINGW_)
+#if defined(_COMPILER_GCC_)
 void __cdecl __MINGW_NOTHROW _fpreset (void);
 void __cdecl __MINGW_NOTHROW fpreset (void);
 #else
@@ -96,15 +96,18 @@ void __cdecl crt_sig_handler(int sig, int num)
     }
 }
 
+// StackOverflowException needs extra stack space to record the backtrace
+// so we keep one around, shared by all threads
+static jl_mutex_t backtrace_lock;
 static jl_ucontext_t collect_backtrace_fiber;
 static jl_ucontext_t error_return_fiber;
-static PCONTEXT error_ctx;
+static PCONTEXT stkerror_ctx;
+static jl_ptls_t stkerror_ptls;
 static int have_backtrace_fiber;
 static void JL_NORETURN start_backtrace_fiber(void)
 {
-    jl_ptls_t ptls = jl_get_ptls_states();
     // collect the backtrace
-    ptls->bt_size = rec_backtrace_ctx(ptls->bt_data, JL_MAX_BT_SIZE, error_ctx, ptls->pgcstack);
+    stkerror_ptls->bt_size = rec_backtrace_ctx(stkerror_ptls->bt_data, JL_MAX_BT_SIZE, stkerror_ctx, stkerror_ptls->pgcstack);
     // switch back to the execution fiber
     jl_setcontext(&error_return_fiber);
     abort();
@@ -133,8 +136,11 @@ void jl_throw_in_ctx(jl_value_t *excpt, PCONTEXT ctxThread)
             ptls->bt_size = rec_backtrace_ctx(ptls->bt_data, JL_MAX_BT_SIZE, ctxThread, ptls->pgcstack);
         }
         else if (have_backtrace_fiber) {
-            error_ctx = ctxThread;
+            JL_LOCK(&backtrace_lock);
+            stkerror_ctx = ctxThread;
+            stkerror_ptls = ptls;
             jl_swapcontext(&error_return_fiber, &collect_backtrace_fiber);
+            JL_UNLOCK_NOGC(&backtrace_lock);
         }
         ptls->sig_exception = excpt;
     }
@@ -155,13 +161,16 @@ HANDLE hMainThread = INVALID_HANDLE_VALUE;
 static void jl_try_deliver_sigint(void)
 {
     jl_ptls_t ptls2 = jl_all_tls_states[0];
+    jl_lock_profile();
     jl_safepoint_enable_sigint();
     jl_wake_libuv();
     if ((DWORD)-1 == SuspendThread(hMainThread)) {
         // error
         jl_safe_printf("error: SuspendThread failed\n");
+        jl_unlock_profile();
         return;
     }
+    jl_unlock_profile();
     int force = jl_check_force_sigint();
     if (force || (!ptls2->defer_signal && ptls2->io_wait)) {
         jl_safepoint_consume_sigint();
@@ -326,11 +335,14 @@ static DWORD WINAPI profile_bt( LPVOID lparam )
         hBtThread = 0;
         return 0;
     }
+    timeBeginPeriod(tc.wPeriodMin);
     while (1) {
-        if (bt_size_cur < bt_size_max) {
-            DWORD timeout = nsecprof/GIGA;
-            timeout = min(max(timeout, tc.wPeriodMin*2), tc.wPeriodMax/2);
-            Sleep(timeout);
+        DWORD timeout = nsecprof / GIGA;
+        timeout += tc.wPeriodMin;
+        Sleep(timeout);
+        if (bt_size_cur < bt_size_max && running) {
+            JL_LOCK_NOGC(&jl_in_stackwalk);
+            jl_lock_profile();
             if ((DWORD)-1 == SuspendThread(hMainThread)) {
                 fputs("failed to suspend main thread. aborting profiling.", stderr);
                 break;
@@ -341,25 +353,35 @@ static DWORD WINAPI profile_bt( LPVOID lparam )
                 ctxThread.ContextFlags = CONTEXT_CONTROL | CONTEXT_INTEGER;
                 if (!GetThreadContext(hMainThread, &ctxThread)) {
                     fputs("failed to get context from main thread. aborting profiling.", stderr);
-                    break;
+                    running = 0;
+                }
+                else {
+                    // Get backtrace data
+                    bt_size_cur += rec_backtrace_ctx((jl_bt_element_t*)bt_data_prof + bt_size_cur,
+                            bt_size_max - bt_size_cur - 1, &ctxThread, NULL);
+                    // Mark the end of this block with 0
+                    if (bt_size_cur < bt_size_max)
+                        bt_data_prof[bt_size_cur++].uintptr = 0;
                 }
-                // Get backtrace data
-                bt_size_cur += rec_backtrace_ctx((jl_bt_element_t*)bt_data_prof + bt_size_cur,
-                    bt_size_max - bt_size_cur - 1, &ctxThread, NULL);
-                // Mark the end of this block with 0
-                bt_data_prof[bt_size_cur].uintptr = 0;
-                bt_size_cur++;
             }
+            jl_unlock_profile();
+            JL_UNLOCK_NOGC(&jl_in_stackwalk);
             if ((DWORD)-1 == ResumeThread(hMainThread)) {
+                timeEndPeriod(tc.wPeriodMin);
                 fputs("failed to resume main thread! aborting.", stderr);
                 gc_debug_critical_error();
                 abort();
             }
         }
         else {
+            timeEndPeriod(tc.wPeriodMin);
             SuspendThread(GetCurrentThread());
+            timeBeginPeriod(tc.wPeriodMin);
         }
     }
+    jl_unlock_profile();
+    JL_UNLOCK_NOGC(&jl_in_stackwalk);
+    timeEndPeriod(tc.wPeriodMin);
     hBtThread = 0;
     return 0;
 }
@@ -379,7 +401,7 @@ JL_DLLEXPORT int jl_profile_start_timer(void)
     }
     else {
         if ((DWORD)-1 == ResumeThread(hBtThread)) {
-            fputs("failed to resume profiling thread.",stderr);
+            fputs("failed to resume profiling thread.", stderr);
             return -2;
         }
     }
@@ -420,5 +442,6 @@ void jl_install_thread_signal_handler(jl_ptls_t ptls)
     collect_backtrace_fiber.uc_stack.ss_sp = (void*)stk;
     collect_backtrace_fiber.uc_stack.ss_size = ssize;
     jl_makecontext(&collect_backtrace_fiber, start_backtrace_fiber);
+    JL_MUTEX_INIT(&backtrace_lock);
     have_backtrace_fiber = 1;
 }
