@@ -1,13 +1,12 @@
-// This file is a part of Julia. License is MIT: http://julialang.org/license
+// This file is a part of Julia. License is MIT: https://julialang.org/license
 
-#ifndef LIBSUPPORT_H
-#define LIBSUPPORT_H
+#ifndef JL_LIBSUPPORT_H
+#define JL_LIBSUPPORT_H
 
 #include "platform.h"
 
 #include <stdlib.h>
 #include <stdarg.h>
-#include <assert.h>
 #include "dtypes.h"
 #include "utils.h"
 #include "utf8.h"
@@ -23,7 +22,7 @@
 extern "C" {
 #endif
 
-DLLEXPORT void libsupport_init(void);
+JL_DLLEXPORT void libsupport_init(void);
 
 #ifdef __cplusplus
 }
