@@ -509,7 +509,7 @@ size_t u8_vprintf(const char *fmt, va_list ap)
     if ((intptr_t)cnt < 0)
         return 0;
     if (cnt >= sz) {
-        buf = (char*)malloc(cnt + 1);
+        buf = (char*)malloc_s(cnt+1);
         needfree = 1;
         vsnprintf(buf, cnt+1, fmt, ap);
     }
@@ -517,7 +517,8 @@ size_t u8_vprintf(const char *fmt, va_list ap)
     nc = u8_toucs(wcs, cnt+1, buf, cnt);
     wcs[nc] = 0;
     printf("%ls", (wchar_t*)wcs);
-    if (needfree) free(buf);
+    if (needfree)
+        free(buf);
     return nc;
 }
 
