@@ -52,9 +52,7 @@ static inline void sanitizer_start_switch_fiber(const void* bottom, size_t size)
 static inline void sanitizer_finish_switch_fiber(void) {}
 #endif
 
-#if defined(_OS_WINDOWS_)
-volatile int jl_in_stackwalk = 0;
-#else
+#if !defined(_OS_WINDOWS_)
 #ifdef JL_HAVE_UCONTEXT
 #include <ucontext.h>
 #endif
@@ -125,7 +123,7 @@ static void NOINLINE save_stack(jl_ptls_t ptls, jl_task_t *lastt, jl_task_t **pt
     else {
         buf = lastt->stkbuf;
     }
-    *pt = lastt; // clear the gc-root for the target task before copying the stack for saving
+    *pt = NULL; // clear the gc-root for the target task before copying the stack for saving
     lastt->copy_stack = nb;
     lastt->sticky = 1;
     memcpy_a16((uint64_t*)buf, (uint64_t*)frame_addr, nb);
@@ -248,10 +246,24 @@ JL_DLLEXPORT void julia_init(JL_IMAGE_SEARCH rel)
     _julia_init(rel);
 }
 
+JL_DLLEXPORT void jl_set_next_task(jl_task_t *task)
+{
+    jl_get_ptls_states()->next_task = task;
+}
+
+JL_DLLEXPORT jl_task_t *jl_get_next_task(void)
+{
+    jl_ptls_t ptls = jl_get_ptls_states();
+    if (ptls->next_task)
+        return ptls->next_task;
+    return ptls->current_task;
+}
+
 void jl_release_task_stack(jl_ptls_t ptls, jl_task_t *task);
 
-static void ctx_switch(jl_ptls_t ptls, jl_task_t **pt)
+static void ctx_switch(jl_ptls_t ptls)
 {
+    jl_task_t **pt = &ptls->next_task;
     jl_task_t *t = *pt;
     assert(t != ptls->current_task);
     jl_task_t *lastt = ptls->current_task;
@@ -283,7 +295,7 @@ static void ctx_switch(jl_ptls_t ptls, jl_task_t **pt)
     }
 
     if (killed) {
-        *pt = lastt; // can't fail after here: clear the gc-root for the target task now
+        *pt = NULL; // can't fail after here: clear the gc-root for the target task now
         lastt->gcstack = NULL;
         if (!lastt->copy_stack && lastt->stkbuf) {
             // early free of stkbuf back to the pool
@@ -302,7 +314,7 @@ static void ctx_switch(jl_ptls_t ptls, jl_task_t **pt)
         }
         else
 #endif
-        *pt = lastt; // can't fail after here: clear the gc-root for the target task now
+        *pt = NULL; // can't fail after here: clear the gc-root for the target task now
         lastt->gcstack = ptls->pgcstack;
     }
 
@@ -366,10 +378,10 @@ static jl_ptls_t NOINLINE refetch_ptls(void)
     return jl_get_ptls_states();
 }
 
-JL_DLLEXPORT void jl_switchto(jl_task_t **pt)
+JL_DLLEXPORT void jl_switch(void)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
-    jl_task_t *t = *pt;
+    jl_task_t *t = ptls->next_task;
     jl_task_t *ct = ptls->current_task;
     if (t == ct) {
         return;
@@ -401,7 +413,7 @@ JL_DLLEXPORT void jl_switchto(jl_task_t **pt)
         jl_timing_block_stop(blk);
 #endif
 
-    ctx_switch(ptls, pt);
+    ctx_switch(ptls);
 
 #ifdef MIGRATE_TASKS
     ptls = refetch_ptls();
@@ -432,6 +444,12 @@ JL_DLLEXPORT void jl_switchto(jl_task_t **pt)
         jl_sigint_safepoint(ptls);
 }
 
+JL_DLLEXPORT void jl_switchto(jl_task_t **pt)
+{
+    jl_set_next_task(*pt);
+    jl_switch();
+}
+
 JL_DLLEXPORT JL_NORETURN void jl_no_exc_handler(jl_value_t *e)
 {
     jl_printf(JL_STDERR, "fatal: error thrown and no exception handler available.\n");
@@ -568,7 +586,7 @@ JL_DLLEXPORT jl_task_t *jl_new_task(jl_function_t *start, jl_value_t *completion
     t->prio = -1;
     t->tid = -1;
 #ifdef ENABLE_TIMINGS
-    t->timing_stack = NULL;
+    t->timing_stack = jl_root_timing;
 #endif
     arraylist_new(&t->locks, 0);
 
