@@ -32,6 +32,7 @@ JL_DLLEXPORT int jl_lineno = 0; // need to update jl_critical_error if this is T
 JL_DLLEXPORT const char *jl_filename = "none"; // need to update jl_critical_error if this is TLS
 
 htable_t jl_current_modules;
+jl_mutex_t jl_modules_mutex;
 
 JL_DLLEXPORT void jl_add_standard_imports(jl_module_t *m)
 {
@@ -49,7 +50,6 @@ void jl_init_main_module(void)
 
     jl_main_module = jl_new_module(jl_symbol("Main"));
     jl_main_module->parent = jl_main_module;
-    jl_core_module->parent = jl_main_module;
     jl_set_const(jl_main_module, jl_symbol("Core"),
                  (jl_value_t*)jl_core_module);
     jl_set_global(jl_core_module, jl_symbol("Main"),
@@ -108,6 +108,12 @@ jl_array_t *jl_get_loaded_modules(void)
     return NULL;
 }
 
+static int jl_is__toplevel__mod(jl_module_t *mod)
+{
+    return jl_base_module &&
+        (jl_value_t*)mod == jl_get_global(jl_base_module, jl_symbol("__toplevel__"));
+}
+
 // TODO: add locks around global state mutation operations
 jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
 {
@@ -116,6 +122,11 @@ jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
     if (jl_array_len(ex->args) != 3 || !jl_is_expr(jl_exprarg(ex, 2))) {
         jl_error("syntax: malformed module expression");
     }
+
+    if (((jl_expr_t *)(jl_exprarg(ex, 2)))->head != jl_symbol("block")) {
+        jl_error("syntax: module expression third argument must be a block");
+    }
+
     int std_imports = (jl_exprarg(ex, 0) == jl_true);
     jl_sym_t *name = (jl_sym_t*)jl_exprarg(ex, 1);
     if (!jl_is_symbol(name)) {
@@ -125,33 +136,38 @@ jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
     jl_module_t *newm = jl_new_module(name);
     jl_value_t *form = (jl_value_t*)newm;
     JL_GC_PUSH1(&form);
+    JL_LOCK(&jl_modules_mutex);
     ptrhash_put(&jl_current_modules, (void*)newm, (void*)((uintptr_t)HT_NOTFOUND + 1));
+    JL_UNLOCK(&jl_modules_mutex);
 
     // copy parent environment info into submodule
     newm->uuid = parent_module->uuid;
-    if (jl_base_module &&
-            (jl_value_t*)parent_module == jl_get_global(jl_base_module, jl_symbol("__toplevel__"))) {
+    if (jl_is__toplevel__mod(parent_module)) {
         newm->parent = newm;
         jl_register_root_module(newm);
     }
     else {
+        newm->parent = parent_module;
         jl_binding_t *b = jl_get_binding_wr(parent_module, name, 1);
         jl_declare_constant(b);
-        if (b->value != NULL) {
-            if (!jl_is_module(b->value)) {
+        jl_value_t *old = jl_atomic_compare_exchange(&b->value, NULL, (jl_value_t*)newm);
+        if (old != NULL) {
+            if (!jl_is_module(old)) {
                 jl_errorf("invalid redefinition of constant %s", jl_symbol_name(name));
             }
-            if (jl_generating_output()) {
+            if (jl_generating_output())
                 jl_errorf("cannot replace module %s during compilation", jl_symbol_name(name));
-            }
             jl_printf(JL_STDERR, "WARNING: replacing module %s.\n", jl_symbol_name(name));
-            // create a hidden gc root for the old module
-            uintptr_t *refcnt = (uintptr_t*)ptrhash_bp(&jl_current_modules, (void*)b->value);
-            *refcnt += 1;
+            old = jl_atomic_exchange(&b->value, (jl_value_t*)newm);
         }
-        newm->parent = parent_module;
-        b->value = (jl_value_t*)newm;
         jl_gc_wb_binding(b, newm);
+        if (old != NULL) {
+            // create a hidden gc root for the old module
+            JL_LOCK(&jl_modules_mutex);
+            uintptr_t *refcnt = (uintptr_t*)ptrhash_bp(&jl_current_modules, (void*)old);
+            *refcnt += 1;
+            JL_UNLOCK(&jl_modules_mutex);
+        }
     }
 
     if (parent_module == jl_main_module && name == jl_symbol("Base")) {
@@ -204,6 +220,7 @@ jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
     }
 #endif
 
+    JL_LOCK(&jl_modules_mutex);
     uintptr_t *refcnt = (uintptr_t*)ptrhash_bp(&jl_current_modules, (void*)newm);
     assert(*refcnt > (uintptr_t)HT_NOTFOUND);
     *refcnt -= 1;
@@ -216,6 +233,7 @@ jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
     // defer init of children until parent is done being defined
     // then initialize all in definition-finished order
     // at build time, don't run them at all (defer for runtime)
+    form = NULL;
     if (!jl_generating_output()) {
         if (!ptrhash_has(&jl_current_modules, (void*)newm->parent)) {
             size_t i, l = jl_array_len(jl_module_init_order);
@@ -232,12 +250,16 @@ jl_value_t *jl_eval_module_expr(jl_module_t *parent_module, jl_expr_t *ex)
             }
             if (ns < l)
                 jl_array_del_end(jl_module_init_order, l - ns);
-            l = jl_array_len(form);
-            for (i = 0; i < l; i++) {
-                jl_module_t *m = (jl_module_t*)jl_array_ptr_ref(form, i);
-                JL_GC_PROMISE_ROOTED(m);
-                jl_module_run_initializer(m);
-            }
+        }
+    }
+    JL_UNLOCK(&jl_modules_mutex);
+
+    if (form) {
+        size_t i, l = jl_array_len(form);
+        for (i = 0; i < l; i++) {
+            jl_module_t *m = (jl_module_t*)jl_array_ptr_ref(form, i);
+            JL_GC_PROMISE_ROOTED(m);
+            jl_module_run_initializer(m);
         }
     }
 
@@ -303,8 +325,7 @@ static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs)
         *has_defs = 1;
         return;
     }
-    else if (head == method_sym || head == abstracttype_sym || head == primtype_sym ||
-             head == structtype_sym || jl_is_toplevel_only_expr(v)) {
+    else if (head == method_sym || jl_is_toplevel_only_expr(v)) {
         *has_defs = 1;
     }
     else if (head == cfunction_sym) {
@@ -330,10 +351,15 @@ static void expr_attributes(jl_value_t *v, int *has_intrinsics, int *has_defs)
         else if (jl_is_quotenode(f)) {
             called = jl_quotenode_value(f);
         }
-        if (called && jl_is_intrinsic(called) && jl_unbox_int32(called) == (int)llvmcall) {
-            *has_intrinsics = 1;
-            return;
+        if (called) {
+            if (jl_is_intrinsic(called) && jl_unbox_int32(called) == (int)llvmcall) {
+                *has_intrinsics = 1;
+            }
+            if (called == jl_builtin__typebody) {
+                *has_defs = 1;
+            }
         }
+        return;
     }
     int i;
     for (i = 0; i < jl_array_len(e->args); i++) {
@@ -631,7 +657,7 @@ jl_value_t *jl_toplevel_eval_flex(jl_module_t *JL_NONNULL m, jl_value_t *e, int
     size_t last_age = ptls->world_age;
     if (!expanded && jl_needs_lowering(e)) {
         ptls->world_age = jl_world_counter;
-        ex = (jl_expr_t*)jl_expand_with_loc(e, m, jl_filename, jl_lineno);
+        ex = (jl_expr_t*)jl_expand_with_loc_warn(e, m, jl_filename, jl_lineno);
         ptls->world_age = last_age;
     }
     jl_sym_t *head = jl_is_expr(ex) ? ex->head : NULL;
@@ -823,30 +849,46 @@ JL_DLLEXPORT jl_value_t *jl_toplevel_eval(jl_module_t *m, jl_value_t *v)
     return jl_toplevel_eval_flex(m, v, 1, 0);
 }
 
+// Check module `m` is open for `eval/include`, or throw an error.
+void jl_check_open_for(jl_module_t *m, const char* funcname)
+{
+    if (jl_options.incremental && jl_generating_output()) {
+        if (m != jl_main_module) { // TODO: this was grand-fathered in
+            JL_LOCK(&jl_modules_mutex);
+            int open = ptrhash_has(&jl_current_modules, (void*)m);
+            JL_UNLOCK(&jl_modules_mutex);
+            if (!open && !jl_is__toplevel__mod(m)) {
+                const char* name = jl_symbol_name(m->name);
+                jl_errorf("Evaluation into the closed module `%s` breaks incremental compilation "
+                          "because the side effects will not be permanent. "
+                          "This is likely due to some other module mutating `%s` with `%s` during "
+                          "precompilation - don't do this.", name, name, funcname);
+            }
+        }
+    }
+}
+
 JL_DLLEXPORT jl_value_t *jl_toplevel_eval_in(jl_module_t *m, jl_value_t *ex)
 {
     jl_ptls_t ptls = jl_get_ptls_states();
     if (ptls->in_pure_callback)
         jl_error("eval cannot be used in a generated function");
+    jl_check_open_for(m, "eval");
     jl_value_t *v = NULL;
     int last_lineno = jl_lineno;
-    if (jl_options.incremental && jl_generating_output()) {
-        if (!ptrhash_has(&jl_current_modules, (void*)m)) {
-            if (m != jl_main_module) { // TODO: this was grand-fathered in
-                jl_printf(JL_STDERR, "WARNING: eval into closed module %s:\n", jl_symbol_name(m->name));
-                jl_static_show(JL_STDERR, ex);
-                jl_printf(JL_STDERR, "\n  ** incremental compilation may be fatally broken for this module **\n\n");
-            }
-        }
-    }
+    const char *last_filename = jl_filename;
+    jl_lineno = 1;
+    jl_filename = "none";
     JL_TRY {
         v = jl_toplevel_eval(m, ex);
     }
     JL_CATCH {
         jl_lineno = last_lineno;
+        jl_filename = last_filename;
         jl_rethrow();
     }
     jl_lineno = last_lineno;
+    jl_filename = last_filename;
     assert(v);
     return v;
 }
@@ -863,13 +905,18 @@ JL_DLLEXPORT jl_value_t *jl_infer_thunk(jl_code_info_t *thk, jl_module_t *m)
     return (jl_value_t*)jl_any_type;
 }
 
-JL_DLLEXPORT jl_value_t *jl_load(jl_module_t *module, const char *fname)
+JL_DLLEXPORT jl_value_t *jl_load_rewrite(jl_module_t *module, const char *fname, jl_value_t *mapexpr)
 {
     uv_stat_t stbuf;
     if (jl_stat(fname, (char*)&stbuf) != 0 || (stbuf.st_mode & S_IFMT) != S_IFREG) {
         jl_errorf("could not open file %s", fname);
     }
-    return jl_parse_eval_all(fname, NULL, 0, module);
+    return jl_parse_eval_all(fname, NULL, 0, module, mapexpr);
+}
+
+JL_DLLEXPORT jl_value_t *jl_load(jl_module_t *module, const char *fname)
+{
+    return jl_load_rewrite(module, fname, NULL);
 }
 
 // load from filename given as a String object
