@@ -181,210 +181,60 @@ static inline int sig_match_simple(jl_value_t *arg1, jl_value_t **args, size_t n
 
 // ----- MethodCache helper functions ----- //
 
-static inline size_t jl_intref(const jl_array_t *arr, size_t idx) JL_NOTSAFEPOINT
+// predicate to fast-test if this type is a leaf type that can exist in the cache
+// and does not need a more expensive linear scan to find all intersections
+// be careful not to put non-leaf types or DataType/UnionAll in the cache here,
+// since they should have a lower priority and need to go into the sorted list
+int is_cache_leaf(jl_value_t *ty)
 {
-    jl_value_t *el = jl_tparam0(jl_typeof(arr));
-    if (el == (jl_value_t*)jl_uint8_type)
-        return ((uint8_t*)jl_array_data(arr))[idx];
-    else if (el == (jl_value_t*)jl_uint16_type)
-        return ((uint16_t*)jl_array_data(arr))[idx];
-    else if (el == (jl_value_t*)jl_uint32_type)
-        return ((uint32_t*)jl_array_data(arr))[idx];
-    else
-        abort();
+    return (jl_is_concrete_type(ty) && !jl_is_kind(ty));
 }
 
-static inline void jl_intset(const jl_array_t *arr, size_t idx, size_t val) JL_NOTSAFEPOINT
+static jl_typemap_t **mtcache_hash_lookup_bp(jl_array_t *cache JL_PROPAGATES_ROOT, jl_value_t *ty) JL_NOTSAFEPOINT
 {
-    jl_value_t *el = jl_tparam0(jl_typeof(arr));
-    if (el == (jl_value_t*)jl_uint8_type)
-        ((uint8_t*)jl_array_data(arr))[idx] = val;
-    else if (el == (jl_value_t*)jl_uint16_type)
-        ((uint16_t*)jl_array_data(arr))[idx] = val;
-    else if (el == (jl_value_t*)jl_uint32_type)
-        ((uint32_t*)jl_array_data(arr))[idx] = val;
-    else
-        abort();
+    if (cache == (jl_array_t*)jl_an_empty_vec_any)
+        return NULL;
+    jl_typemap_t **pml = jl_table_peek_bp(cache, ty);
+    JL_GC_PROMISE_ROOTED(pml); // clang-sa doesn't trust our JL_PROPAGATES_ROOT claim
+    return pml;
 }
 
-static inline size_t jl_max_int(const jl_array_t *arr)
+static void mtcache_hash_insert(jl_array_t **cache, jl_value_t *parent, jl_value_t *key, jl_typemap_t *val)
 {
-    jl_value_t *el = jl_tparam0(jl_typeof(arr));
-    if (el == (jl_value_t*)jl_uint8_type)
-        return 0xFF;
-    else if (el == (jl_value_t*)jl_uint16_type)
-        return 0xFFFF;
-    else if (el == (jl_value_t*)jl_uint32_type)
-        return 0xFFFFFFFF;
-    else
-        abort();
+    int inserted = 0;
+    jl_array_t *a = *cache;
+    if (a == (jl_array_t*)jl_an_empty_vec_any) {
+        a = jl_alloc_vec_any(16);
+        *cache = a;
+        jl_gc_wb(parent, a);
+    }
+    a = jl_eqtable_put(a, key, val, &inserted);
+    assert(inserted);
+    if (a != *cache) {
+        *cache = a;
+        jl_gc_wb(parent, a);
+    }
 }
 
-
-static jl_array_t *jl_alloc_int_1d(size_t np, size_t len)
+static jl_typemap_t *mtcache_hash_lookup(jl_array_t *cache JL_PROPAGATES_ROOT, jl_value_t *ty) JL_NOTSAFEPOINT
 {
-    jl_value_t *ty;
-    if (np < 0xFF) {
-        ty = jl_array_uint8_type;
-    }
-    else if (np < 0xFFFF) {
-        static jl_value_t *int16 JL_ALWAYS_LEAFTYPE = NULL;
-        if (int16 == NULL)
-            int16 = jl_apply_array_type((jl_value_t*)jl_uint16_type, 1);
-        ty = int16;
-    }
-    else {
-        assert(np < 0x7FFFFFFF);
-        static jl_value_t *int32 JL_ALWAYS_LEAFTYPE = NULL;
-        if (int32 == NULL)
-            int32 = jl_apply_array_type((jl_value_t*)jl_uint32_type, 1);
-        ty = int32;
-    }
-    jl_array_t *a = jl_alloc_array_1d(ty, len);
-    memset(a->data, 0, len * a->elsize);
-    return a;
-}
-
-static inline
-jl_typemap_t *mtcache_hash_lookup(const struct jl_ordereddict_t *a JL_PROPAGATES_ROOT, jl_value_t *ty, int8_t tparam, int8_t offs) JL_NOTSAFEPOINT
-{
-    uintptr_t uid = ((jl_datatype_t*)ty)->uid;
-    jl_typemap_t *ml = jl_nothing;
-    if (!uid)
-        return ml;
-    size_t idx = jl_intref(a->indices, uid & (a->indices->nrows-1));
-    if (idx > 0) {
-        ml = jl_array_ptr_ref(a->values, idx - 1);
-        if (ml == jl_nothing)
-            return ml;
-        jl_value_t *t;
-        if (jl_typeof(ml) == (jl_value_t*)jl_typemap_level_type) {
-            t = ((jl_typemap_level_t*)ml)->key;
-        }
-        else {
-            t = jl_tparam(jl_unwrap_unionall(jl_typemap_entry_sig(ml)), offs);
-            if (tparam)
-                t = jl_tparam0(t);
-        }
-        if (t != ty)
-            ml = jl_nothing;
-    }
+    if (cache == (jl_array_t*)jl_an_empty_vec_any)
+        return NULL;
+    jl_typemap_t *ml = (jl_typemap_t*)jl_eqtable_get(cache, ty, jl_nothing);
+    JL_GC_PROMISE_ROOTED(ml); // clang-sa doesn't trust our JL_PROPAGATES_ROOT claim
     return ml;
 }
 
-static void mtcache_rehash(struct jl_ordereddict_t *pa, size_t newlen, jl_value_t *parent, int8_t tparam, int8_t offs)
-{
-    size_t i, nval = jl_array_len(pa->values);
-    jl_array_t *n = jl_alloc_int_1d(nval + 1, newlen);
-    for (i = 1; i <= nval; i++) {
-        jl_typemap_t *ml = jl_array_ptr_ref(pa->values, i - 1);
-        if (ml == jl_nothing)
-            continue;
-        jl_value_t *t;
-        if (jl_typeof(ml) == (jl_value_t*)jl_typemap_level_type) {
-            t = ((jl_typemap_level_t*)ml)->key;
-        }
-        else {
-            t = jl_tparam(jl_unwrap_unionall(jl_typemap_entry_sig(ml)), offs);
-            if (tparam)
-                t = jl_tparam0(t);
-        }
-        uintptr_t uid = ((jl_datatype_t*)t)->uid;
-        size_t newi = uid & (newlen - 1);
-        if (jl_intref(n, newi) == 0) {
-            jl_intset(n, newi, i);
-        }
-        else {
-            // hash collision: start over after doubling the size again
-            i = 0;
-            newlen *= 2;
-            n = jl_alloc_int_1d(nval + 1, newlen);
-        }
-    }
-    pa->indices = n;
-    jl_gc_wb(parent, n);
-}
-
-// Recursively rehash a TypeMap (for example, after deserialization)
-void jl_typemap_rehash(jl_typemap_t *ml, int8_t offs);
-void jl_typemap_rehash_array(struct jl_ordereddict_t *pa, jl_value_t *parent, int8_t tparam, int8_t offs)
-{
-    size_t i, len = jl_array_len(pa->values);
-    for (i = 0; i < len; i++) {
-        jl_typemap_t *ml = jl_array_ptr_ref(pa->values, i);
-        assert(ml != NULL);
-        jl_typemap_rehash(ml, offs+1);
-    }
-    mtcache_rehash(pa, 4 * next_power_of_two(len), parent, tparam, offs);
-}
-void jl_typemap_rehash(jl_typemap_t *ml, int8_t offs)
-{
-    if (jl_typeof(ml) == (jl_value_t*)jl_typemap_level_type) {
-        jl_typemap_level_t *node = (jl_typemap_level_t*)ml;
-        if (node->targ.values != (void *)jl_nothing)
-            jl_typemap_rehash_array(&node->targ, ml, 1, offs);
-        if (node->arg1.values != (void *)jl_nothing)
-            jl_typemap_rehash_array(&node->arg1, ml, 0, offs);
-        jl_typemap_rehash(node->any, offs + 1);
-    }
-}
-
-static jl_typemap_t **mtcache_hash_bp(struct jl_ordereddict_t *pa JL_PROPAGATES_ROOT, jl_value_t *ty,
-                                     int8_t tparam, int8_t offs, jl_value_t *parent)
-{
-    if (jl_is_datatype(ty)) {
-        uintptr_t uid = ((jl_datatype_t*)ty)->uid;
-        if (!uid || jl_is_kind(ty) || jl_has_free_typevars(ty))
-            // be careful not to put non-leaf types or DataType/UnionAll in the cache here,
-            // since they should have a lower priority and need to go into the sorted list
-            return NULL;
-        if (pa->values == (void*)jl_nothing) {
-            pa->indices = jl_alloc_int_1d(0, INIT_CACHE_SIZE);
-            jl_gc_wb(parent, pa->indices);
-            pa->values = jl_alloc_vec_any(0);
-            jl_gc_wb(parent, pa->values);
-        }
-        while (1) {
-            size_t slot = uid & (pa->indices->nrows - 1);
-            size_t idx = jl_intref(pa->indices, slot);
-            if (idx == 0) {
-                jl_array_ptr_1d_push(pa->values, jl_nothing);
-                idx = jl_array_len(pa->values);
-                if (idx > jl_max_int(pa->indices))
-                    mtcache_rehash(pa, jl_array_len(pa->indices), parent, tparam, offs);
-                jl_intset(pa->indices, slot, idx);
-                return &((jl_typemap_t **)jl_array_ptr_data(pa->values))[idx - 1];
-            }
-            jl_typemap_t **pml = &((jl_typemap_t **)jl_array_ptr_data(pa->values))[idx - 1];
-            if (*pml == jl_nothing)
-                return pml;
-            jl_value_t *t;
-            if (jl_typeof(*pml) == (jl_value_t*)jl_typemap_level_type) {
-                t = ((jl_typemap_level_t*)*pml)->key;
-            }
-            else {
-                t = jl_tparam(jl_unwrap_unionall(
-                    jl_typemap_entry_sig(*pml)),
-                    offs);
-                if (tparam)
-                    t = jl_tparam0(t);
-            }
-            if (t == ty)
-                return pml;
-            mtcache_rehash(pa, jl_array_len(pa->indices) * 2, parent, tparam, offs);
-        }
-    }
-    return NULL;
-}
-
 // ----- Sorted Type Signature Lookup Matching ----- //
 
-static int jl_typemap_array_visitor(struct jl_ordereddict_t *a, jl_typemap_visitor_fptr fptr, void *closure)
+static int jl_typemap_array_visitor(jl_array_t *a, jl_typemap_visitor_fptr fptr, void *closure)
 {
-    size_t i, l = jl_array_len(a->values);
-    jl_typemap_t **data = (jl_typemap_t **)jl_array_ptr_data(a->values);
-    for(i=0; i < l; i++) {
-        if (!jl_typemap_visitor(data[i], fptr, closure))
+    size_t i, l = jl_array_len(a);
+    jl_typemap_t **data = (jl_typemap_t **)jl_array_data(a);
+    for (i = 1; i < l; i += 2) {
+        jl_value_t *d = data[i];
+        JL_GC_PROMISE_ROOTED(d);
+        if (d && !jl_typemap_visitor(d, fptr, closure))
             return 0;
     }
     return 1;
@@ -405,11 +255,11 @@ int jl_typemap_visitor(jl_typemap_t *cache, jl_typemap_visitor_fptr fptr, void *
 {
     if (jl_typeof(cache) == (jl_value_t*)jl_typemap_level_type) {
         jl_typemap_level_t *node = (jl_typemap_level_t*)cache;
-        if (node->targ.values != (void*)jl_nothing)
-            if (!jl_typemap_array_visitor(&node->targ, fptr, closure))
+        if (node->targ != (jl_array_t*)jl_an_empty_vec_any)
+            if (!jl_typemap_array_visitor(node->targ, fptr, closure))
                 return 0;
-        if (node->arg1.values != (void*)jl_nothing)
-            if (!jl_typemap_array_visitor(&node->arg1, fptr, closure))
+        if (node->arg1 != (jl_array_t*)jl_an_empty_vec_any)
+            if (!jl_typemap_array_visitor(node->arg1, fptr, closure))
                 return 0;
         if (!jl_typemap_node_visitor(node->linear, fptr, closure))
             return 0;
@@ -420,36 +270,23 @@ int jl_typemap_visitor(jl_typemap_t *cache, jl_typemap_visitor_fptr fptr, void *
     }
 }
 
-// predicate to fast-test if this type is a leaf type that can exist in the cache
-// and does not need a more expensive linear scan to find all intersections
-int is_cache_leaf(jl_value_t *ty)
-{
-    return (jl_is_datatype(ty) && ((jl_datatype_t*)ty)->uid != 0 && !jl_is_kind(ty));
-}
-
-static int jl_typemap_intersection_array_visitor(struct jl_ordereddict_t *a, jl_value_t *ty, int tparam,
+static int jl_typemap_intersection_array_visitor(jl_array_t *a, jl_value_t *ty, int tparam,
                                                  int offs, struct typemap_intersection_env *closure)
 {
-    size_t i, l = jl_array_len(a->values);
-    jl_typemap_t **data = (jl_typemap_t **)jl_array_ptr_data(a->values);
-    for (i = 0; i < l; i++) {
-        jl_typemap_t *ml = data[i];
-        if (ml == jl_nothing)
+    size_t i, l = jl_array_len(a);
+    jl_typemap_t **data = (jl_typemap_t **)jl_array_data(a);
+    for (i = 0; i < l; i += 2) {
+        jl_value_t *t = data[i];
+        JL_GC_PROMISE_ROOTED(t);
+        if (t == jl_nothing || t == NULL)
             continue;
-        jl_value_t *t;
-        if (jl_typeof(ml) == (jl_value_t *)jl_typemap_level_type) {
-            t = ((jl_typemap_level_t*)ml)->key;
-        }
-        else {
-            t = jl_tparam(jl_unwrap_unionall(jl_typemap_entry_sig(ml)), offs);
-            if (tparam)
-                t = jl_tparam0(t);
-        }
         // `t` is a leaftype, so intersection test becomes subtype
         if (ty == (jl_value_t*)jl_any_type || // easy case: Any always matches
             (tparam
              ? (jl_typeof(t) == ty || jl_isa(t, ty)) // (Type{t} <: ty), where is_leaf_type(t) => isa(t, ty)
              : (t == ty || jl_subtype(t, ty)))) {
+            jl_typemap_t *ml = data[i + 1];
+            JL_GC_PROMISE_ROOTED(ml);
             if (!jl_typemap_intersection_visitor(ml, offs + 1, closure))
                 return 0;
         }
@@ -505,14 +342,13 @@ int jl_typemap_intersection_visitor(jl_typemap_t *map, int offs,
     //TODO: fast-path for leaf-type tuples?
     //if (ttypes->isdispatchtuple) {
     //    register jl_typemap_intersection_visitor_fptr fptr = closure->fptr;
-    //    size_t world = jl_world_counter;
-    //    while (world > 0) {
-    //        jl_typemap_entry_t *ml = jl_typemap_assoc_by_type(map, ttypes, &closure->env, /*subtype*/1, offs, world, /*max_world_mask*/(~(size_t)0) >> 1);
-    //        if (!ml)
-    //            break;
-    //        if (!fptr(ml, closure))
-    //            return 0;
-    //        world = ml->min_world - 1;
+    //        struct jl_typemap_assoc search = {(jl_value_t*)closure->type, world, closure->env, 0, ~(size_t)0};
+    //        jl_typemap_entry_t *ml = jl_typemap_assoc_by_type(map, search, offs, /*subtype*/1);
+    //        if (ml) {
+    //            closure->env = search->env;
+    //            if (!fptr(ml, closure))
+    //                return 0;
+    //        }
     //    }
     //    return 1;
     //}
@@ -537,12 +373,12 @@ int jl_typemap_intersection_visitor(jl_typemap_t *map, int offs,
                 else
                     ty = (jl_value_t*)jl_any_type;
             }
-            if (cache->targ.values != (void*)jl_nothing) {
+            if (cache->targ != (jl_array_t*)jl_an_empty_vec_any) {
                 jl_value_t *typetype = jl_is_type_type(ty) ? jl_tparam0(ty) : NULL;
                 if (typetype) {
                     if (is_cache_leaf(typetype)) {
                         // direct lookup of leaf types
-                        jl_typemap_t *ml = mtcache_hash_lookup(&cache->targ, typetype, 1, offs);
+                        jl_typemap_t *ml = mtcache_hash_lookup(cache->targ, typetype);
                         if (ml != jl_nothing) {
                             if (!jl_typemap_intersection_visitor(ml, offs+1, closure)) return 0;
                         }
@@ -552,20 +388,20 @@ int jl_typemap_intersection_visitor(jl_typemap_t *map, int offs,
                     // else an array scan is required to check subtypes
                     // first, fast-path: optimized pre-intersection test to see if `ty` could intersect with any Type
                     if (typetype || !jl_has_empty_intersection((jl_value_t*)jl_type_type, ty))
-                        if (!jl_typemap_intersection_array_visitor(&cache->targ, ty, 1, offs, closure)) return 0;
+                        if (!jl_typemap_intersection_array_visitor(cache->targ, ty, 1, offs, closure)) return 0;
                 }
             }
-            if (cache->arg1.values != (void*)jl_nothing) {
+            if (cache->arg1 != (jl_array_t*)jl_an_empty_vec_any) {
                 if (is_cache_leaf(ty)) {
                     // direct lookup of leaf types
-                    jl_typemap_t *ml = mtcache_hash_lookup(&cache->arg1, ty, 0, offs);
+                    jl_typemap_t *ml = mtcache_hash_lookup(cache->arg1, ty);
                     if (ml != jl_nothing) {
                         if (!jl_typemap_intersection_visitor(ml, offs+1, closure)) return 0;
                     }
                 }
                 else {
                     // else an array scan is required to check subtypes
-                    if (!jl_typemap_intersection_array_visitor(&cache->arg1, ty, 0, offs, closure)) return 0;
+                    if (!jl_typemap_intersection_array_visitor(cache->arg1, ty, 0, offs, closure)) return 0;
                 }
             }
         }
@@ -591,16 +427,16 @@ int jl_typemap_intersection_visitor(jl_typemap_t *map, int offs,
   (the function) is always the same for most functions.
 */
 static jl_typemap_entry_t *jl_typemap_entry_assoc_by_type(
-        jl_typemap_entry_t *ml, jl_value_t *types,
-        jl_svec_t **penv, size_t world, size_t max_world_mask)
+        jl_typemap_entry_t *ml,
+        struct jl_typemap_assoc *search)
 {
+    jl_value_t *types = search->types;
+    JL_GC_PROMISE_ROOTED(types);
     jl_value_t *unw = jl_unwrap_unionall((jl_value_t*)types);
     int isua = jl_is_unionall(types);
     size_t n = jl_nparams(unw);
     int typesisva = n == 0 ? 0 : jl_is_vararg_type(jl_tparam(unw, n-1));
     for (; ml != (void*)jl_nothing; ml = ml->next) {
-        if (world < ml->min_world || world > (ml->max_world | max_world_mask))
-            continue; // ignore replaced methods
         size_t lensig = jl_nparams(jl_unwrap_unionall((jl_value_t*)ml->sig));
         if (lensig == n || (ml->va && lensig <= n+1)) {
             int resetenv = 0, ismatch = 1;
@@ -623,8 +459,8 @@ static jl_typemap_entry_t *jl_typemap_entry_assoc_by_type(
                 ismatch = sig_match_by_type_simple(jl_svec_data(((jl_datatype_t*)types)->parameters), n,
                                                    ml->sig, lensig, ml->va);
             else {
-                ismatch = jl_subtype_matching(types, (jl_value_t*)ml->sig, penv);
-                if (ismatch && penv)
+                ismatch = jl_subtype_matching(types, (jl_value_t*)ml->sig, search->env ? &search->env : NULL);
+                if (ismatch && search->env)
                     resetenv = 1;
             }
 
@@ -637,11 +473,29 @@ static jl_typemap_entry_t *jl_typemap_entry_assoc_by_type(
                         break;
                     }
                 }
-                if (ismatch)
-                    return ml;
+                if (ismatch) {
+                    if (search->world < ml->min_world) {
+                        // ignore method table entries that are part of a later world
+                        if (search->max_valid >= ml->min_world)
+                            search->max_valid = ml->min_world - 1;
+                    }
+                    else if (search->world > ml->max_world) {
+                        // ignore method table entries that have been replaced in the current world
+                        if (search->min_valid <= ml->max_world)
+                            search->min_valid = ml->max_world + 1;
+                    }
+                    else {
+                        // intersect the env valid range with method's valid range
+                        if (search->min_valid < ml->min_world)
+                            search->min_valid = ml->min_world;
+                        if (search->max_valid > ml->max_world)
+                            search->max_valid = ml->max_world;
+                        return ml;
+                    }
+                }
             }
             if (resetenv)
-                *penv = jl_emptysvec;
+                search->env = jl_emptysvec;
         }
     }
     return NULL;
@@ -650,13 +504,14 @@ static jl_typemap_entry_t *jl_typemap_entry_assoc_by_type(
 int jl_obviously_unequal(jl_value_t *a, jl_value_t *b);
 
 static jl_typemap_entry_t *jl_typemap_entry_lookup_by_type(
-        jl_typemap_entry_t *ml, jl_value_t *types,
-        size_t world, size_t max_world_mask)
+        jl_typemap_entry_t *ml, struct jl_typemap_assoc *search)
 {
     for (; ml != (void*)jl_nothing; ml = ml->next) {
-        if (world < ml->min_world || world > (ml->max_world | max_world_mask))
+        if (search->world < ml->min_world || search->world > ml->max_world)
             continue;
         // unroll the first few cases here, to the extent that is possible to do fast and easily
+        jl_value_t *types = search->types;
+        JL_GC_PROMISE_ROOTED(types);
         jl_value_t *a = jl_unwrap_unionall(types);
         jl_value_t *b = jl_unwrap_unionall((jl_value_t*)ml->sig);
         size_t na = jl_nparams(a);
@@ -679,7 +534,7 @@ static jl_typemap_entry_t *jl_typemap_entry_lookup_by_type(
                 }
             }
         }
-        if (jl_types_equal((jl_value_t*)types, (jl_value_t*)ml->sig))
+        if (jl_types_equal(types, (jl_value_t*)ml->sig))
             return ml;
     }
     return NULL;
@@ -689,14 +544,16 @@ static jl_typemap_entry_t *jl_typemap_entry_lookup_by_type(
 // this is the general entry point for looking up a type in the cache
 // as a subtype, or with type_equal
 jl_typemap_entry_t *jl_typemap_assoc_by_type(
-        jl_typemap_t *ml_or_cache, jl_value_t *types, jl_svec_t **penv,
-        int8_t subtype, int8_t offs, size_t world, size_t max_world_mask)
+        jl_typemap_t *ml_or_cache,
+        struct jl_typemap_assoc *search,
+        int8_t offs, uint8_t subtype)
 {
     if (jl_typeof(ml_or_cache) == (jl_value_t *)jl_typemap_level_type) {
         jl_typemap_level_t *cache = (jl_typemap_level_t*)ml_or_cache;
         // called object is the primary key for constructors, otherwise first argument
         jl_value_t *ty = NULL;
-        jl_value_t *ttypes = jl_unwrap_unionall((jl_value_t*)types);
+        jl_value_t *ttypes = jl_unwrap_unionall((jl_value_t*)search->types);
+        JL_GC_PROMISE_ROOTED(ttypes);
         assert(jl_is_datatype(ttypes));
         size_t l = jl_nparams(ttypes);
         int isva = 0;
@@ -717,48 +574,50 @@ jl_typemap_entry_t *jl_typemap_assoc_by_type(
         // If there is a type at offs, look in the optimized caches
         if (!subtype) {
             if (ty && jl_is_any(ty))
-                return jl_typemap_assoc_by_type(cache->any, types, penv, subtype, offs + 1, world, max_world_mask);
+                return jl_typemap_assoc_by_type(cache->any, search, offs + 1, subtype);
             if (isva) // in lookup mode, want to match Vararg exactly, not as a subtype
                 ty = NULL;
         }
         if (ty) {
             if (jl_is_type_type(ty)) {
                 jl_value_t *a0 = jl_tparam0(ty);
-                if (cache->targ.values != (void*)jl_nothing && jl_is_datatype(a0)) {
-                    jl_typemap_t *ml = mtcache_hash_lookup(&cache->targ, a0, 1, offs);
+                if (is_cache_leaf(a0)) {
+                    if (cache->targ != (jl_array_t*)jl_an_empty_vec_any) {
+                        jl_typemap_t *ml = mtcache_hash_lookup(cache->targ, a0);
+                        if (ml != jl_nothing) {
+                            jl_typemap_entry_t *li = jl_typemap_assoc_by_type(ml, search, offs + 1, subtype);
+                            if (li) return li;
+                        }
+                    }
+                    if (!subtype) return NULL;
+                }
+            }
+            if (is_cache_leaf(ty)) {
+                if (cache->arg1 != (jl_array_t*)jl_an_empty_vec_any) {
+                    jl_typemap_t *ml = mtcache_hash_lookup(cache->arg1, ty);
                     if (ml != jl_nothing) {
-                        jl_typemap_entry_t *li =
-                            jl_typemap_assoc_by_type(ml, types, penv, subtype, offs + 1, world, max_world_mask);
+                        jl_typemap_entry_t *li = jl_typemap_assoc_by_type(ml, search, offs + 1, subtype);
                         if (li) return li;
                     }
                 }
-                if (!subtype && is_cache_leaf(a0)) return NULL;
+                if (!subtype) return NULL;
             }
-            if (cache->arg1.values != (void*)jl_nothing && jl_is_datatype(ty)) {
-                jl_typemap_t *ml = mtcache_hash_lookup(&cache->arg1, ty, 0, offs);
-                if (ml != jl_nothing) {
-                    jl_typemap_entry_t *li =
-                        jl_typemap_assoc_by_type(ml, types, penv, subtype, offs + 1, world, max_world_mask);
-                    if (li) return li;
-                }
-            }
-            if (!subtype && is_cache_leaf(ty)) return NULL;
         }
         // Always check the list (since offs doesn't always start at 0)
         if (subtype) {
-            jl_typemap_entry_t *li = jl_typemap_entry_assoc_by_type(cache->linear, types, penv, world, max_world_mask);
+            jl_typemap_entry_t *li = jl_typemap_entry_assoc_by_type(cache->linear, search);
             if (li) return li;
-            return jl_typemap_assoc_by_type(cache->any, types, penv, subtype, offs + 1, world, max_world_mask);
+            return jl_typemap_assoc_by_type(cache->any, search, offs + 1, subtype);
         }
         else {
-            return jl_typemap_entry_lookup_by_type(cache->linear, types, world, max_world_mask);
+            return jl_typemap_entry_lookup_by_type(cache->linear, search);
         }
     }
     else {
         jl_typemap_entry_t *leaf = (jl_typemap_entry_t*)ml_or_cache;
         return subtype ?
-            jl_typemap_entry_assoc_by_type(leaf, types, penv, world, max_world_mask) :
-            jl_typemap_entry_lookup_by_type(leaf, types, world, max_world_mask);
+            jl_typemap_entry_assoc_by_type(leaf, search) :
+            jl_typemap_entry_lookup_by_type(leaf, search);
     }
 }
 
@@ -846,13 +705,13 @@ jl_typemap_entry_t *jl_typemap_level_assoc_exact(jl_typemap_level_t *cache, jl_v
         jl_value_t *a1 = (offs == 0 ? arg1 : args[offs - 1]);
         jl_value_t *ty = jl_typeof(a1);
         assert(jl_is_datatype(ty));
-        if (ty == (jl_value_t*)jl_datatype_type && cache->targ.values != (void*)jl_nothing) {
-            jl_typemap_t *ml_or_cache = mtcache_hash_lookup(&cache->targ, a1, 1, offs);
+        if (ty == (jl_value_t*)jl_datatype_type && cache->targ != (jl_array_t*)jl_an_empty_vec_any && is_cache_leaf(a1)) {
+            jl_typemap_t *ml_or_cache = mtcache_hash_lookup(cache->targ, a1);
             jl_typemap_entry_t *ml = jl_typemap_assoc_exact(ml_or_cache, arg1, args, n, offs+1, world);
             if (ml) return ml;
         }
-        if (cache->arg1.values != (void*)jl_nothing) {
-            jl_typemap_t *ml_or_cache = mtcache_hash_lookup(&cache->arg1, ty, 0, offs);
+        if (cache->arg1 != (jl_array_t*)jl_an_empty_vec_any && is_cache_leaf(ty)) {
+            jl_typemap_t *ml_or_cache = mtcache_hash_lookup(cache->arg1, ty);
             jl_typemap_entry_t *ml = jl_typemap_assoc_exact(ml_or_cache, arg1, args, n, offs+1, world);
             if (ml) return ml;
         }
@@ -890,22 +749,18 @@ static jl_typemap_level_t *jl_new_typemap_level(void)
     jl_typemap_level_t *cache =
         (jl_typemap_level_t*)jl_gc_alloc(ptls, sizeof(jl_typemap_level_t),
                                          jl_typemap_level_type);
-    cache->key = NULL;
     cache->linear = (jl_typemap_entry_t*)jl_nothing;
     cache->any = jl_nothing;
-    cache->targ.indices = (jl_array_t*)jl_nothing;
-    cache->targ.values = (jl_array_t*)jl_nothing;
-    cache->arg1.indices = (jl_array_t*)jl_nothing;
-    cache->arg1.values = (jl_array_t*)jl_nothing;
+    cache->targ = (jl_array_t*)jl_an_empty_vec_any;
+    cache->arg1 = (jl_array_t*)jl_an_empty_vec_any;
     return cache;
 }
 
 static jl_typemap_level_t *jl_method_convert_list_to_cache(
-        jl_typemap_t *map, jl_typemap_entry_t *ml, jl_value_t *key, int8_t offs,
+        jl_typemap_t *map, jl_typemap_entry_t *ml, int8_t offs,
         const struct jl_typemap_info *tparams)
 {
     jl_typemap_level_t *cache = jl_new_typemap_level();
-    cache->key = key;
     jl_typemap_entry_t *next = NULL;
     JL_GC_PUSH3(&cache, &next, &ml);
     while (ml != (void*)jl_nothing) {
@@ -937,7 +792,7 @@ static void jl_typemap_list_insert_(
 
 static void jl_typemap_insert_generic(
         jl_typemap_t *map, jl_typemap_t **pml, jl_value_t *parent,
-        jl_typemap_entry_t *newrec, jl_value_t *key, int8_t offs,
+        jl_typemap_entry_t *newrec, int8_t offs,
         const struct jl_typemap_info *tparams)
 {
     if (jl_typeof(*pml) == (jl_value_t*)jl_typemap_level_type) {
@@ -949,7 +804,7 @@ static void jl_typemap_insert_generic(
     if (count > MAX_METHLIST_COUNT) {
         *pml = (jl_typemap_t*)jl_method_convert_list_to_cache(
             map, (jl_typemap_entry_t *)*pml,
-            key, offs, tparams);
+            offs, tparams);
         jl_gc_wb(parent, *pml);
         jl_typemap_level_insert_(map, (jl_typemap_level_t*)*pml, newrec, offs, tparams);
         return;
@@ -960,14 +815,18 @@ static void jl_typemap_insert_generic(
 }
 
 static int jl_typemap_array_insert_(
-        jl_typemap_t *map, struct jl_ordereddict_t *cache, jl_value_t *key, jl_typemap_entry_t *newrec,
+        jl_typemap_t *map, jl_array_t **cache, jl_value_t *key, jl_typemap_entry_t *newrec,
         jl_value_t *parent, int8_t tparam, int8_t offs,
         const struct jl_typemap_info *tparams)
 {
-    jl_typemap_t **pml = mtcache_hash_bp(cache, key, tparam, offs, (jl_value_t*)parent);
-    if (pml)
-        jl_typemap_insert_generic(map, pml, (jl_value_t*)cache->values, newrec, key, offs+1, tparams);
-    return pml != NULL;
+    if (!is_cache_leaf(key))
+        return 0;
+    jl_typemap_t **pml = mtcache_hash_lookup_bp(*cache, key);
+    if (pml != NULL)
+        jl_typemap_insert_generic(map, pml, (jl_value_t*)*cache, newrec, offs+1, tparams);
+    else
+        mtcache_hash_insert(cache, parent, key, (jl_typemap_t*)newrec);
+    return 1;
 }
 
 static void jl_typemap_level_insert_(
@@ -994,7 +853,7 @@ static void jl_typemap_level_insert_(
     }
     // If the type at `offs` is Any, put it in the Any list
     if (t1 && jl_is_any(t1)) {
-        jl_typemap_insert_generic(map, &cache->any, (jl_value_t*)cache, newrec, (jl_value_t*)jl_any_type, offs+1, tparams);
+        jl_typemap_insert_generic(map, &cache->any, (jl_value_t*)cache, newrec, offs+1, tparams);
         return;
     }
     // Don't put Varargs in the optimized caches (too hard to handle in lookup and bp)
@@ -1057,7 +916,7 @@ jl_typemap_entry_t *jl_typemap_insert(jl_typemap_t **cache, jl_value_t *parent,
             newrec->isleafsig = newrec->issimplesig = 0;
     }
     // TODO: assert that guardsigs == jl_emptysvec && simplesig == jl_nothing if isleafsig and optimize with that knowledge?
-    jl_typemap_insert_generic(*cache, cache, parent, newrec, NULL, offs, tparams);
+    jl_typemap_insert_generic(*cache, cache, parent, newrec, offs, tparams);
     JL_GC_POP();
     return newrec;
 }
