@@ -7,10 +7,10 @@ struct DateLocale
     months_abbr::Vector{String}
     days_of_week::Vector{String}
     days_of_week_abbr::Vector{String}
-    month_value::Dict{String, Int}
-    month_abbr_value::Dict{String, Int}
-    day_of_week_value::Dict{String, Int}
-    day_of_week_abbr_value::Dict{String, Int}
+    month_value::Dict{String, Int64}
+    month_abbr_value::Dict{String, Int64}
+    day_of_week_value::Dict{String, Int64}
+    day_of_week_abbr_value::Dict{String, Int64}
 end
 
 function locale_dict(names::Vector{<:AbstractString})
@@ -654,11 +654,15 @@ function quarterofyear(dt::TimeType)
     m = month(dt)
     return m < 4 ? 1 : m < 7 ? 2 : m < 10 ? 3 : 4
 end
-const QUARTERDAYS = (0, 90, 181, 273)
+
+const QUARTERDAYS = (0, 31, 59, 0, 30, 61, 0, 31, 62, 0, 31, 61)
 
 """
     dayofquarter(dt::TimeType) -> Int
 
 Return the day of the current quarter of `dt`. Range of value is 1:92.
 """
-dayofquarter(dt::TimeType) = dayofyear(dt) - QUARTERDAYS[quarterofyear(dt)]
+function dayofquarter(dt::TimeType)
+    (y, m, d) = yearmonthday(dt)
+    return QUARTERDAYS[m] + d + (m == 3 && isleapyear(y))
+end
