@@ -171,7 +171,22 @@ end
     @test y + Dates.Year(1f0) == Dates.Year(2)
     @test y * 4 == Dates.Year(4)
     @test y * 4f0 == Dates.Year(4)
-    @test_throws InexactError y * 3//4 == Dates.Year(1)
+    @test Dates.Year(2) * 0.5 == y
+    @test Dates.Year(2) * 3//2 == Dates.Year(3)
+    @test_throws InexactError y * 0.5
+    @test_throws InexactError y * 3//4
+    @test (1:1:5)*Second(5) === Second(5)*(1:1:5) === Second(5):Second(5):Second(25) === (1:5)*Second(5)
+    @test collect(1:1:5)*Second(5) == Second(5)*collect(1:1:5) == (1:5)*Second(5)
+    @test (Second(2):Second(2):Second(10))/Second(2) === 1.0:1.0:5.0
+    @test collect(Second(2):Second(2):Second(10))/Second(2) == 1:1:5
+    @test (Second(2):Second(2):Second(10)) / 2 === Second(1):Second(1):Second(5)
+    @test collect(Second(2):Second(2):Second(10)) / 2 == Second(1):Second(1):Second(5)
+    @test Dates.Year(4) / 2 == Dates.Year(2)
+    @test Dates.Year(4) / 2f0 == Dates.Year(2)
+    @test Dates.Year(4) / 0.5 == Dates.Year(8)
+    @test Dates.Year(4) / 2//3 == Dates.Year(6)
+    @test_throws InexactError Dates.Year(4) / 3.0
+    @test_throws InexactError Dates.Year(4) / 3//2
     @test div(y, 2) == Dates.Year(0)
     @test_throws MethodError div(2, y) == Dates.Year(2)
     @test div(y, y) == 1
