@@ -71,6 +71,8 @@ Keyword arguments:
 * `tunnel`: if `true` then SSH tunneling will be used to connect to the worker from the
   master process. Default is `false`.
 
+* `multiplex`: if `true` then SSH multiplexing is used for SSH tunneling. Default is `false`.
+
 * `sshflags`: specifies additional ssh options, e.g. ```sshflags=\`-i /home/foo/bar.pem\````
 
 * `max_parallel`: specifies the maximum number of workers connected to in parallel at a
@@ -113,9 +115,9 @@ This timeout can be controlled via environment variable `JULIA_WORKER_TIMEOUT`.
 The value of `JULIA_WORKER_TIMEOUT` on the master process specifies the number of seconds a
 newly launched worker waits for connection establishment.
 """
-function addprocs(machines::AbstractVector; tunnel=false, sshflags=``, max_parallel=10, kwargs...)
+function addprocs(machines::AbstractVector; tunnel=false, multiplex=false, sshflags=``, max_parallel=10, kwargs...)
     check_addprocs_args(kwargs)
-    addprocs(SSHManager(machines); tunnel=tunnel, sshflags=sshflags, max_parallel=max_parallel, kwargs...)
+    addprocs(SSHManager(machines); tunnel=tunnel, multiplex=multiplex, sshflags=sshflags, max_parallel=max_parallel, kwargs...)
 end
 
 
@@ -142,13 +144,52 @@ function launch(manager::SSHManager, params::Dict, launched::Array, launch_ntfy:
 end
 
 
-show(io::IO, manager::SSHManager) = println(io, "SSHManager(machines=", manager.machines, ")")
+Base.show(io::IO, manager::SSHManager) = print(io, "SSHManager(machines=", manager.machines, ")")
 
 
-function launch_on_machine(manager::SSHManager, machine, cnt, params, launched, launch_ntfy::Condition)
+function parse_machine(machine::AbstractString)
+    hoststr = ""
+    portnum = nothing
+
+    if machine[begin] == '['  # ipv6 bracket notation (RFC 2732)
+        ipv6_end = findlast(']', machine)
+        if ipv6_end == nothing
+            throw(ArgumentError("invalid machine definition format string: invalid port format \"$machine_def\""))
+        end
+        hoststr = machine[begin+1 : prevind(machine,ipv6_end)]
+        machine_def = split(machine[ipv6_end : end] , ':')
+    else    # ipv4
+        machine_def = split(machine, ':')
+        hoststr = machine_def[1]
+    end
+
+    if length(machine_def) > 2
+        throw(ArgumentError("invalid machine definition format string: invalid port format \"$machine_def\""))
+    end
+
+    if length(machine_def) == 2
+        portstr = machine_def[2]
+
+        portnum = tryparse(Int, portstr)
+        if portnum == nothing
+            msg = "invalid machine definition format string: invalid port format \"$machine_def\""
+            throw(ArgumentError(msg))
+        end
+
+        if portnum < 1 || portnum > 65535
+            msg = "invalid machine definition format string: invalid port number \"$machine_def\""
+            throw(ArgumentError(msg))
+        end
+    end
+    (hoststr, portnum)
+end
+
+function launch_on_machine(manager::SSHManager, machine::AbstractString, cnt, params::Dict, launched::Array, launch_ntfy::Condition)
     dir = params[:dir]
     exename = params[:exename]
     exeflags = params[:exeflags]
+    tunnel = params[:tunnel]
+    multiplex = params[:multiplex]
 
     # machine could be of the format [user@]host[:port] bind_addr[:bind_port]
     # machine format string is split on whitespace
@@ -161,23 +202,24 @@ function launch_on_machine(manager::SSHManager, machine, cnt, params, launched,
     end
     exeflags = `$exeflags --worker`
 
-    machine_def = split(machine_bind[1], ':')
-    # if this machine def has a port number, add the port information to the ssh flags
-    if length(machine_def) > 2
-        throw(ArgumentError("invalid machine definition format string: invalid port format \"$machine_def\""))
-    end
-    host = machine_def[1]
-    portopt = ``
-    if length(machine_def) == 2
-        portstr = machine_def[2]
-        if !all(isdigit, portstr) || (p = parse(Int,portstr); p < 1 || p > 65535)
-            msg = "invalid machine definition format string: invalid port format \"$machine_def\""
-            throw(ArgumentError(msg))
-        end
-        portopt = ` -p $(machine_def[2]) `
-    end
+    host, portnum = parse_machine(machine_bind[1])
+    portopt = portnum === nothing ? `` : `-p $portnum`
     sshflags = `$(params[:sshflags]) $portopt`
 
+    if tunnel
+        # First it checks if ssh multiplexing has been already enabled and the master process is running.
+        # If it's already running, later ssh sessions also use the same ssh multiplexing session even if
+        # `multiplex` is not explicitly specified; otherwise the tunneling session launched later won't
+        # go to background and hang. This is because of OpenSSH implementation.
+        if success(`ssh $sshflags -O check $host`)
+            multiplex = true
+        elseif multiplex
+            # automatically create an SSH multiplexing session at the next SSH connection
+            controlpath = "~/.ssh/julia-%r@%h:%p"
+            sshflags = `$sshflags -o ControlMaster=auto -o ControlPath=$controlpath -o ControlPersist=no`
+        end
+    end
+
     # Build up the ssh command
 
     # the default worker timeout
@@ -211,7 +253,8 @@ function launch_on_machine(manager::SSHManager, machine, cnt, params, launched,
     wconfig = WorkerConfig()
     wconfig.io = io.out
     wconfig.host = host
-    wconfig.tunnel = params[:tunnel]
+    wconfig.tunnel = tunnel
+    wconfig.multiplex = multiplex
     wconfig.sshflags = sshflags
     wconfig.exeflags = exeflags
     wconfig.exename = exename
@@ -256,25 +299,32 @@ end
 
 
 """
-    ssh_tunnel(user, host, bind_addr, port, sshflags) -> localport
+    ssh_tunnel(user, host, bind_addr, port, sshflags, multiplex) -> localport
 
 Establish an SSH tunnel to a remote worker.
 Return a port number `localport` such that `localhost:localport` connects to `host:port`.
 """
-function ssh_tunnel(user, host, bind_addr, port, sshflags)
+function ssh_tunnel(user, host, bind_addr, port, sshflags, multiplex)
     port = Int(port)
     cnt = ntries = 100
-    # if we cannot do port forwarding, bail immediately
+
     # the connection is forwarded to `port` on the remote server over the local port `localport`
-    # the -f option backgrounds the ssh session
-    # `sleep 60` command specifies that an alloted time of 60 seconds is allowed to start the
-    # remote julia process and establish the network connections specified by the process topology.
-    # If no connections are made within 60 seconds, ssh will exit and an error will be printed on the
-    # process that launched the remote process.
-    ssh = `ssh -T -a -x -o ExitOnForwardFailure=yes`
     while cnt > 0
         localport = next_tunnel_port()
-        if success(detach(`$ssh -f $sshflags $user@$host -L $localport:$bind_addr:$port sleep 60`))
+        if multiplex
+            # It assumes that an ssh multiplexing session has been already started by the remote worker.
+            cmd = `ssh $sshflags -O forward -L $localport:$bind_addr:$port $user@$host`
+        else
+            # if we cannot do port forwarding, fail immediately
+            # the -f option backgrounds the ssh session
+            # `sleep 60` command specifies that an alloted time of 60 seconds is allowed to start the
+            # remote julia process and establish the network connections specified by the process topology.
+            # If no connections are made within 60 seconds, ssh will exit and an error will be printed on the
+            # process that launched the remote process.
+            ssh = `ssh -T -a -x -o ExitOnForwardFailure=yes`
+            cmd = detach(`$ssh -f $sshflags $user@$host -L $localport:$bind_addr:$port sleep 60`)
+        end
+        if success(cmd)
             return localport
         end
         cnt -= 1
@@ -316,7 +366,7 @@ function addprocs(np::Integer; restrict=true, kwargs...)
     addprocs(LocalManager(np, restrict); kwargs...)
 end
 
-show(io::IO, manager::LocalManager) = println(io, "LocalManager()")
+Base.show(io::IO, manager::LocalManager) = print(io, "LocalManager()")
 
 function launch(manager::LocalManager, params::Dict, launched::Array, c::Condition)
     dir = params[:dir]
@@ -427,9 +477,11 @@ function connect(manager::ClusterManager, pid::Int, config::WorkerConfig)
         sem = tunnel_hosts_map[pubhost]
 
         sshflags = notnothing(config.sshflags)
+        multiplex = something(config.multiplex, false)
         acquire(sem)
         try
-            (s, bind_addr) = connect_to_worker(pubhost, bind_addr, port, user, sshflags)
+            (s, bind_addr, forward) = connect_to_worker_with_tunnel(pubhost, bind_addr, port, user, sshflags, multiplex)
+            config.forward = forward
         finally
             release(sem)
         end
@@ -459,65 +511,77 @@ function connect_w2w(pid::Int, config::WorkerConfig)
     (s,s)
 end
 
-const client_port = Ref{Cushort}(0)
+const client_port = Ref{UInt16}(0)
 
-function socket_reuse_port()
+function socket_reuse_port(iptype)
     if ccall(:jl_has_so_reuseport, Int32, ()) == 1
-        s = TCPSocket(delay = false)
+        sock = TCPSocket(delay = false)
 
         # Some systems (e.g. Linux) require the port to be bound before setting REUSEPORT
         bind_early = Sys.islinux()
 
-        bind_early && bind_client_port(s)
-        rc = ccall(:jl_tcp_reuseport, Int32, (Ptr{Cvoid},), s.handle)
+        bind_early && bind_client_port(sock, iptype)
+        rc = ccall(:jl_tcp_reuseport, Int32, (Ptr{Cvoid},), sock.handle)
         if rc < 0
+            close(sock)
+
             # This is an issue only on systems with lots of client connections, hence delay the warning
             nworkers() > 128 && @warn "Error trying to reuse client port number, falling back to regular socket" maxlog=1
 
             # provide a clean new socket
             return TCPSocket()
         end
-        bind_early || bind_client_port(s)
-        return s
+        bind_early || bind_client_port(sock, iptype)
+        return sock
     else
         return TCPSocket()
     end
 end
 
-# TODO: this doesn't belong here, it belongs in Sockets
-function bind_client_port(s::TCPSocket)
-    Sockets.iolock_begin()
-    @assert s.status == Sockets.StatusInit
-    host_in = Ref(hton(UInt32(0))) # IPv4 0.0.0.0
-    err = ccall(:jl_tcp_bind, Int32, (Ptr{Cvoid}, UInt16, Ptr{Cvoid}, Cuint, Cint),
-                s, hton(client_port[]), host_in, 0, false)
-    Sockets.iolock_end()
-    uv_error("tcp_bind", err)
-
-    _addr, port = getsockname(s)
-    client_port[] = port
-    return s
+function bind_client_port(sock::TCPSocket, iptype)
+    bind_host = iptype(0)
+    if Sockets.bind(sock, bind_host, client_port[])
+        _addr, port = getsockname(sock)
+        client_port[] = port
+    end
+    return sock
 end
 
 function connect_to_worker(host::AbstractString, port::Integer)
-    s = socket_reuse_port()
-    connect(s, host, UInt16(port))
-
     # Avoid calling getaddrinfo if possible - involves a DNS lookup
     # host may be a stringified ipv4 / ipv6 address or a dns name
     bind_addr = nothing
     try
-        bind_addr = string(parse(IPAddr,host))
+        bind_addr = parse(IPAddr,host)
     catch
-        bind_addr = string(getaddrinfo(host))
+        bind_addr = getaddrinfo(host)
     end
-    (s, bind_addr)
+
+    iptype = typeof(bind_addr)
+    sock = socket_reuse_port(iptype)
+    connect(sock, bind_addr, UInt16(port))
+
+    (sock, string(bind_addr))
 end
 
 
-function connect_to_worker(host::AbstractString, bind_addr::AbstractString, port::Integer, tunnel_user::AbstractString, sshflags)
-    s = connect("localhost", ssh_tunnel(tunnel_user, host, bind_addr, UInt16(port), sshflags))
-    (s, bind_addr)
+function connect_to_worker_with_tunnel(host::AbstractString, bind_addr::AbstractString, port::Integer, tunnel_user::AbstractString, sshflags, multiplex)
+    localport = ssh_tunnel(tunnel_user, host, bind_addr, UInt16(port), sshflags, multiplex)
+    s = connect("localhost", localport)
+    forward = "$localport:$bind_addr:$port"
+    (s, bind_addr, forward)
+end
+
+
+function cancel_ssh_tunnel(config::WorkerConfig)
+    host = notnothing(config.host)
+    sshflags = notnothing(config.sshflags)
+    tunnel = something(config.tunnel, false)
+    multiplex = something(config.multiplex, false)
+    if tunnel && multiplex
+        forward = notnothing(config.forward)
+        run(`ssh $sshflags -O cancel -L $forward $host`)
+    end
 end
 
 
@@ -531,7 +595,12 @@ It should cause the remote worker specified by `pid` to exit.
 on `pid`.
 """
 function kill(manager::ClusterManager, pid::Int, config::WorkerConfig)
-    remote_do(exit, pid) # For TCP based transports this will result in a close of the socket
-                       # at our end, which will result in a cleanup of the worker.
+    remote_do(exit, pid)
+    nothing
+end
+
+function kill(manager::SSHManager, pid::Int, config::WorkerConfig)
+    remote_do(exit, pid)
+    cancel_ssh_tunnel(config)
     nothing
 end
