@@ -58,6 +58,7 @@ Note that many editors are already defined. All of the following commands should
 already work:
 
 - emacs
+- emacsclient
 - vim
 - nvim
 - nano
@@ -70,6 +71,7 @@ already work:
 - Visual Studio Code
 - open
 - pycharm
+- bbedit
 
 # Example:
 
@@ -112,22 +114,23 @@ function define_default_editors()
     define_editor(r".*") do cmd, path, line
         `$cmd $path`
     end
+    define_editor([r"\bemacs", "gedit", r"\bgvim"]) do cmd, path, line
+        `$cmd +$line $path`
+    end
+    # Must check that emacs not running in -t/-nw before regex match for general emacs
     define_editor([
         "vim", "vi", "nvim", "mvim", "nano",
         r"\bemacs\b.*\s(-nw|--no-window-system)\b",
         r"\bemacsclient\b.\s*-(-?nw|t|-?tty)\b"], wait=true) do cmd, path, line
         `$cmd +$line $path`
     end
-    define_editor([r"\bemacs", "gedit", r"\bgvim"]) do cmd, path, line
-        `$cmd +$line $path`
-    end
     define_editor(["textmate", "mate", "kate"]) do cmd, path, line
         `$cmd $path -l $line`
     end
-    define_editor([r"\bsubl", r"\batom", "pycharm"]) do cmd, path, line
+    define_editor([r"\bsubl", r"\batom", "pycharm", "bbedit"]) do cmd, path, line
         `$cmd $path:$line`
     end
-    define_editor("code") do cmd, path, line
+    define_editor(["code", "code-insiders"]) do cmd, path, line
         `$cmd -g $path:$line`
     end
     define_editor(r"\bnotepad++") do cmd, path, line
@@ -226,7 +229,12 @@ edit(m::Module) = edit(pathof(m))
 if Sys.iswindows()
     function less(file::AbstractString, line::Integer)
         pager = shell_split(get(ENV, "PAGER", "more"))
-        g = pager[1] == "more" ? "" : "g"
+        if pager[1] == "more"
+            g = ""
+            line -= 1
+        else
+            g = "g"
+        end
         run(Cmd(`$pager +$(line)$(g) \"$file\"`, windows_verbatim = true))
         nothing
     end
