@@ -646,44 +646,48 @@ function generic_matvecmul!(C::AbstractVector{R}, tA, A::AbstractVecOrMat, B::Ab
 
     @inbounds begin
     if tA == 'T'  # fastest case
-        for k = 1:mA
-            aoffs = (k-1)*Astride
-            if mB == 0
-                s = zero(R)
-            else
+        if nA == 0
+            for k = 1:mA
+                _modify!(_add, false, C, k)
+            end
+        else
+            for k = 1:mA
+                aoffs = (k-1)*Astride
                 s = zero(A[aoffs + 1]*B[1] + A[aoffs + 1]*B[1])
+                for i = 1:nA
+                    s += transpose(A[aoffs+i]) * B[i]
+                end
+                _modify!(_add, s, C, k)
             end
-            for i = 1:nA
-                s += transpose(A[aoffs+i]) * B[i]
-            end
-            _modify!(_add, s, C, k)
         end
     elseif tA == 'C'
-        for k = 1:mA
-            aoffs = (k-1)*Astride
-            if mB == 0
-                s = zero(R)
-            else
+        if nA == 0
+            for k = 1:mA
+                _modify!(_add, false, C, k)
+            end
+        else
+            for k = 1:mA
+                aoffs = (k-1)*Astride
                 s = zero(A[aoffs + 1]*B[1] + A[aoffs + 1]*B[1])
+                for i = 1:nA
+                    s += A[aoffs + i]'B[i]
+                end
+                _modify!(_add, s, C, k)
             end
-            for i = 1:nA
-                s += A[aoffs + i]'B[i]
-            end
-            _modify!(_add, s, C, k)
         end
     else # tA == 'N'
         for i = 1:mA
             if !iszero(_add.beta)
                 C[i] *= _add.beta
             elseif mB == 0
-                C[i] = zero(R)
+                C[i] = false
             else
                 C[i] = zero(A[i]*B[1] + A[i]*B[1])
             end
         end
         for k = 1:mB
             aoffs = (k-1)*Astride
-            b = _add(B[k], 0)
+            b = _add(B[k], false)
             for i = 1:mA
                 C[i] += A[aoffs + i] * b
             end
