@@ -60,13 +60,21 @@ structured_broadcast_alloc(bc, ::Type{<:Diagonal}, ::Type{ElType}, n) where {ElT
 # Bidiagonal is tricky as we need to know if it's upper or lower. The promotion
 # system will return Tridiagonal when there's more than one Bidiagonal, but when
 # there's only one, we need to make figure out upper or lower
-find_bidiagonal() = throw(ArgumentError("could not find Bidiagonal within broadcast expression"))
-find_bidiagonal(a::Bidiagonal, rest...) = a
-find_bidiagonal(bc::Broadcast.Broadcasted, rest...) = find_bidiagonal(find_bidiagonal(bc.args...), rest...)
-find_bidiagonal(x, rest...) = find_bidiagonal(rest...)
+merge_uplos(::Nothing, ::Nothing) = nothing
+merge_uplos(a, ::Nothing) = a
+merge_uplos(::Nothing, b) = b
+merge_uplos(a, b) = a == b ? a : 'T'
+
+find_uplo(a::Bidiagonal) = a.uplo
+find_uplo(a) = nothing
+find_uplo(bc::Broadcasted) = mapreduce(find_uplo, merge_uplos, bc.args, init=nothing)
+
 function structured_broadcast_alloc(bc, ::Type{<:Bidiagonal}, ::Type{ElType}, n) where {ElType}
-    ex = find_bidiagonal(bc)
-    return Bidiagonal(Array{ElType}(undef, n),Array{ElType}(undef, n-1), ex.uplo)
+    uplo = find_uplo(bc)
+    if uplo == 'T'
+        return Tridiagonal(Array{ElType}(undef, n-1), Array{ElType}(undef, n), Array{ElType}(undef, n-1))
+    end
+    return Bidiagonal(Array{ElType}(undef, n),Array{ElType}(undef, n-1), uplo)
 end
 structured_broadcast_alloc(bc, ::Type{<:SymTridiagonal}, ::Type{ElType}, n) where {ElType} =
     SymTridiagonal(Array{ElType}(undef, n),Array{ElType}(undef, n-1))
@@ -91,14 +99,22 @@ isstructurepreserving(bc::Broadcasted) = isstructurepreserving(bc.f, bc.args...)
 isstructurepreserving(::Union{typeof(abs),typeof(big)}, ::StructuredMatrix) = true
 isstructurepreserving(::TypeFuncs, ::StructuredMatrix) = true
 isstructurepreserving(::TypeFuncs, ::Ref{<:Type}, ::StructuredMatrix) = true
+function isstructurepreserving(::typeof(Base.literal_pow), ::Ref{typeof(^)}, ::StructuredMatrix, ::Ref{Val{N}}) where N
+    return N isa Integer && N > 0
+end
 isstructurepreserving(f, args...) = false
 
 _iszero(n::Number) = iszero(n)
 _iszero(x) = x == 0
 fzeropreserving(bc) = (v = fzero(bc); !ismissing(v) && _iszero(v))
-# Very conservatively only allow Numbers and Types in this speculative zero-test pass
+# Like sparse matrices, we assume that the zero-preservation property of a broadcasted
+# expression is stable.  We can test the zero-preservability by applying the function
+# in cases where all other arguments are known scalars against a zero from the structured
+# matrix. If any non-structured matrix argument is not a known scalar, we give up.
 fzero(x::Number) = x
 fzero(::Type{T}) where T = T
+fzero(r::Ref) = r[]
+fzero(t::Tuple{Any}) = t[1]
 fzero(S::StructuredMatrix) = zero(eltype(S))
 fzero(x) = missing
 function fzero(bc::Broadcast.Broadcasted)
