@@ -20,7 +20,15 @@ let n = 10
         A = Areal
         H = UpperHessenberg(A)
         AH = triu(A,-1)
-        @test Matrix(H) == H == AH
+        @test UpperHessenberg(H) === H
+        @test parent(H) === A
+        @test Matrix(H) == Array(H) == H == AH
+        @test real(H) == real(AH)
+        @test real(UpperHessenberg{ComplexF64}(A)) == H
+        @test real(UpperHessenberg{ComplexF64}(H)) == H
+        sim = similar(H, ComplexF64)
+        @test sim isa UpperHessenberg{ComplexF64}
+        @test size(sim) == size(H)
         for x in (2,2+3im)
             @test x*H == H*x == x*AH
             for op in (+,-)
@@ -37,6 +45,16 @@ let n = 10
         H2[5,3]=0
         @test H2 == triu(A2,-1)
         @test_throws ArgumentError H[5,3]=1
+        Hc = UpperHessenberg(Areal + im .* Aimg)
+        AHc = triu(Areal + im .* Aimg,-1)
+        @test real(Hc) == real(AHc)
+        @test imag(Hc) == imag(AHc)
+        @test Array(copy(adjoint(Hc))) == adjoint(Array(Hc))
+        @test Array(copy(transpose(Hc))) == transpose(Array(Hc))
+        @test rmul!(copy(Hc), 2.0) == lmul!(2.0, copy(Hc))
+        H = UpperHessenberg(Areal)
+        @test Array(Hc + H) == Array(Hc) + Array(H)
+        @test Array(Hc - H) == Array(Hc) - Array(H)
     end
 
     @testset for eltya in (Float32, Float64, ComplexF32, ComplexF64, Int), herm in (false, true)
@@ -48,10 +66,12 @@ let n = 10
         A = herm ? Hermitian(A_ + A_') : A_
 
         H = hessenberg(A)
+        @test Hessenberg(H) === H
         eltyh = eltype(H)
         @test size(H.Q, 1) == size(A, 1)
         @test size(H.Q, 2) == size(A, 2)
         @test size(H.Q) == size(A)
+        @test size(H) == size(A)
         @test_throws ErrorException H.Z
         @test convert(Array, H) ≈ A
         @test (H.Q * H.H) * H.Q' ≈ A ≈ (Matrix(H.Q) * Matrix(H.H)) * Matrix(H.Q)'
@@ -59,6 +79,12 @@ let n = 10
         #getindex for HessenbergQ
         @test H.Q[1,1] ≈ Array(H.Q)[1,1]
 
+        # REPL show
+        hessstring = sprint((t, s) -> show(t, "text/plain", s), H)
+        qstring = sprint((t, s) -> show(t, "text/plain", s), H.Q)
+        hstring = sprint((t, s) -> show(t, "text/plain", s), H.H)
+        @test hessstring == "$(summary(H))\nQ factor:\n$qstring\nH factor:\n$hstring"
+
         #iterate
         q,h = H
         @test q == H.Q
@@ -101,4 +127,10 @@ let A = [0.5 0.1 0.9 0.4; 0.9 0.7 0.5 0.4; 0.3 0.4 0.9 0.0; 0.4 0.0 0.0 0.5]
     @test logdet(hessenberg(A)) ≈ logdet(A) ≈ -3.5065578973199822
 end
 
+@testset "Base.propertynames" begin
+    F =  hessenberg([4. 9. 7.; 4. 4. 1.; 4. 3. 2.])
+    @test Base.propertynames(F) == (:Q, :H, :μ)
+    @test Base.propertynames(F, true) == (:Q, :H, :μ, :τ, :factors, :uplo)
+end
+
 end # module TestHessenberg
