@@ -17,8 +17,8 @@ end
     init_data = Profile.len_data()
     while iter < n_tries && Profile.len_data() == init_data
         iter += 1
-        tend = time() + t
-        while time() < tend end
+        tend = time_ns() + 1e9 * t
+        while time_ns() < tend end
     end
 end
 
@@ -93,3 +93,43 @@ end
     @test delay_ == 0.0005
     Profile.init(n=1_000_000, delay=def_delay)
 end
+
+@testset "Line number correction" begin
+    @profile busywait(1, 20)
+    _, fdict0 = Profile.flatten(Profile.retrieve()...)
+    Base.update_stackframes_callback[] = function(list)
+        modify((sf, n)) = sf.func == :busywait ? (StackTraces.StackFrame(sf.func, sf.file, sf.line+2, sf.linfo, sf.from_c, sf.inlined, sf.pointer), n) : (sf, n)
+        map!(modify, list, list)
+    end
+    _, fdictc = Profile.flatten(Profile.retrieve()...)
+    Base.update_stackframes_callback[] = identity
+    function getline(sfs)
+        for sf in sfs
+            sf.func == :busywait && return sf.line
+        end
+        nothing
+    end
+    @test getline(values(fdictc)) == getline(values(fdict0)) + 2
+end
+
+# Profile deadlocking in compilation (debuginfo registration)
+let cmd = Base.julia_cmd()
+    script = """
+        using Profile
+        f(::Val) = GC.safepoint()
+        @profile for i = 1:10^3; f(Val(i)); end
+        print(Profile.len_data())
+        """
+    p = open(`$cmd -e $script`)
+    t = Timer(120) do t
+        # should be under 10 seconds, so give it 2 minutes then report failure
+        println("KILLING BY PROFILE TEST WATCHDOG\n")
+        kill(p, Base.SIGTERM)
+        sleep(10)
+        kill(p, Base.SIGKILL)
+    end
+    s = read(p, String)
+    close(t)
+    @test success(p)
+    @test parse(Int, s) > 1000
+end
