@@ -8,7 +8,7 @@ using ..Terminals
 import ..Terminals: raw!, width, height, cmove, getX,
                        getY, clear_line, beep
 
-import Base: ensureroom, peek, show, AnyDict, position
+import Base: ensureroom, show, AnyDict, position
 using Base: something
 
 abstract type TextInterface end
@@ -480,7 +480,7 @@ function refresh_multi_line(termbuf::TerminalBuffer, terminal::UnixTerminal, buf
     return InputAreaState(cur_row, curs_row)
 end
 
-function highlight_region(lwrite::String, regstart::Int, regstop::Int, written::Int, slength::Int)
+function highlight_region(lwrite::AbstractString, regstart::Int, regstop::Int, written::Int, slength::Int)
     if written <= regstop <= written+slength
         i = thisind(lwrite, regstop-written)
         lwrite = lwrite[1:i] * Base.disable_text_style[:reverse] * lwrite[nextind(lwrite, i):end]
@@ -2218,6 +2218,7 @@ const prefix_history_keymap = merge!(
             match_input(map, s, IOBuffer(c))(s, keymap_data(ps, mode(s)))
         end,
         # match escape sequences for pass through
+        "^x*" => "*",
         "\e*" => "*",
         "\e[*" => "*",
         "\eO*"  => "*",
