@@ -1,7 +1,20 @@
 # This file is a part of Julia. License is MIT: https://julialang.org/license
 
+"""
+Run Evaluate Print Loop (REPL)
+
+    Example minimal code
+    ```
+    import REPL
+    term = REPL.Terminals.TTYTerminal("dumb", stdin, stdout, stderr)
+    repl = REPL.LineEditREPL(term, true)
+    REPL.run_repl(repl)
+    ```
+"""
 module REPL
 
+Base.Experimental.@optlevel 1
+
 using Base.Meta, Sockets
 import InteractiveUtils
 
@@ -66,12 +79,44 @@ mutable struct REPLBackend
     response_channel::Channel
     "flag indicating the state of this backend"
     in_eval::Bool
+    "transformation functions to apply before evaluating expressions"
+    ast_transforms::Vector{Any}
     "current backend task"
     backend_task::Task
 
-    REPLBackend(repl_channel, response_channel, in_eval) =
-        new(repl_channel, response_channel, in_eval)
+    REPLBackend(repl_channel, response_channel, in_eval, ast_transforms=copy(repl_ast_transforms)) =
+        new(repl_channel, response_channel, in_eval, ast_transforms)
 end
+REPLBackend() = REPLBackend(Channel(1),Channel(1),false)
+
+"""
+    softscope(ex)
+
+Return a modified version of the parsed expression `ex` that uses
+the REPL's "soft" scoping rules for global syntax blocks.
+"""
+function softscope(@nospecialize ex)
+    if ex isa Expr
+        h = ex.head
+        if h === :toplevel
+            ex′ = Expr(h)
+            map!(softscope, resize!(ex′.args, length(ex.args)), ex.args)
+            return ex′
+        elseif h in (:meta, :import, :using, :export, :module, :error, :incomplete, :thunk)
+            return ex
+        elseif h === :global && all(x->isa(x, Symbol), ex.args)
+            return ex
+        else
+            return Expr(:block, Expr(:softscope, true), ex)
+        end
+    end
+    return ex
+end
+
+# Temporary alias until Documenter updates
+const softscope! = softscope
+
+const repl_ast_transforms = Any[softscope] # defaults for new REPL backends
 
 function eval_user_input(@nospecialize(ast), backend::REPLBackend)
     lasterr = nothing
@@ -83,6 +128,9 @@ function eval_user_input(@nospecialize(ast), backend::REPLBackend)
                 put!(backend.response_channel, (lasterr,true))
             else
                 backend.in_eval = true
+                for xf in backend.ast_transforms
+                    ast = Base.invokelatest(xf, ast)
+                end
                 value = Core.eval(Main, ast)
                 backend.in_eval = false
                 # note: use jl_set_global to make sure value isn't passed through `expand`
@@ -102,24 +150,53 @@ function eval_user_input(@nospecialize(ast), backend::REPLBackend)
     nothing
 end
 
+"""
+    start_repl_backend(repl_channel::Channel,response_channel::Channel)
+
+    Starts loop for REPL backend
+    Returns a REPLBackend with backend_task assigned
+
+    Deprecated since sync / async behavior cannot be selected
+"""
 function start_repl_backend(repl_channel::Channel, response_channel::Channel)
+    # Maintain legacy behavior of asynchronous backend
     backend = REPLBackend(repl_channel, response_channel, false)
-    backend.backend_task = @async begin
-        # include looks at this to determine the relative include path
-        # nothing means cwd
-        while true
-            tls = task_local_storage()
-            tls[:SOURCE_PATH] = nothing
-            ast, show_value = take!(backend.repl_channel)
-            if show_value == -1
-                # exit flag
-                break
-            end
-            eval_user_input(ast, backend)
-        end
-    end
+    # Assignment will be made twice, but will be immediately available
+    backend.backend_task = @async start_repl_backend(backend)
     return backend
 end
+
+"""
+    start_repl_backend(backend::REPLBackend)
+
+    Call directly to run backend loop on current Task.
+    Use @async for run backend on new Task.
+
+    Does not return backend until loop is finished.
+"""
+function start_repl_backend(backend::REPLBackend,  @nospecialize(consumer = x -> nothing))
+    backend.backend_task = Base.current_task()
+    consumer(backend)
+    repl_backend_loop(backend)
+    return backend
+end
+
+function repl_backend_loop(backend::REPLBackend)
+    # include looks at this to determine the relative include path
+    # nothing means cwd
+    while true
+        tls = task_local_storage()
+        tls[:SOURCE_PATH] = nothing
+        ast, show_value = take!(backend.repl_channel)
+        if show_value == -1
+            # exit flag
+            break
+        end
+        eval_user_input(ast, backend)
+    end
+    nothing
+end
+
 struct REPLDisplay{R<:AbstractREPL} <: AbstractDisplay
     repl::R
 end
@@ -128,11 +205,11 @@ end
 
 function display(d::REPLDisplay, mime::MIME"text/plain", x)
     io = outstream(d.repl)
+    io = IOContext(io, :limit => true, :module => Main)
     get(io, :color, false) && write(io, answer_color(d.repl))
     if isdefined(d.repl, :options) && isdefined(d.repl.options, :iocontext)
         # this can override the :limit property set initially
-        io = foldl(IOContext, d.repl.options.iocontext,
-                   init=IOContext(io, :limit => true, :module => Main))
+        io = foldl(IOContext, d.repl.options.iocontext, init=io)
     end
     show(io, mime, x)
     println(io)
@@ -192,18 +269,31 @@ function print_response(errio::IO, @nospecialize(response), show_value::Bool, ha
     nothing
 end
 
-# A reference to a backend
+# A reference to a backend that is not mutable
 struct REPLBackendRef
     repl_channel::Channel
     response_channel::Channel
 end
+REPLBackendRef(backend::REPLBackend) = REPLBackendRef(backend.repl_channel,backend.response_channel)
 
-function run_repl(repl::AbstractREPL, @nospecialize(consumer = x -> nothing))
-    repl_channel = Channel(1)
-    response_channel = Channel(1)
-    backend = start_repl_backend(repl_channel, response_channel)
-    consumer(backend)
-    run_frontend(repl, REPLBackendRef(repl_channel, response_channel))
+"""
+    run_repl(repl::AbstractREPL)
+    run_repl(repl, consumer = backend->nothing; backend_on_current_task = true)
+
+    Main function to start the REPL
+
+    consumer is an optional function that takes a REPLBackend as an argument
+"""
+function run_repl(repl::AbstractREPL, @nospecialize(consumer = x -> nothing); backend_on_current_task::Bool = true)
+    backend = REPLBackend()
+    backend_ref = REPLBackendRef(backend)
+    if backend_on_current_task
+        @async run_frontend(repl, backend_ref)
+        start_repl_backend(backend,consumer)
+    else
+        @async start_repl_backend(backend,consumer)
+        run_frontend(repl, backend_ref)
+    end
     return backend
 end
 
@@ -216,6 +306,7 @@ mutable struct BasicREPL <: AbstractREPL
 end
 
 outstream(r::BasicREPL) = r.terminal
+hascolor(r::BasicREPL) = hascolor(r.terminal)
 
 function run_frontend(repl::BasicREPL, backend::REPLBackendRef)
     d = REPLDisplay(repl)
@@ -342,13 +433,14 @@ mutable struct LineEditREPL <: AbstractREPL
     interface::ModalInterface
     backendref::REPLBackendRef
     LineEditREPL(t,hascolor,prompt_color,input_color,answer_color,shell_color,help_color,history_file,in_shell,in_help,envcolors) =
-        new(t,true,prompt_color,input_color,answer_color,shell_color,help_color,history_file,in_shell,
+        new(t,hascolor,prompt_color,input_color,answer_color,shell_color,help_color,history_file,in_shell,
             in_help,envcolors,false,nothing, Options(), nothing)
 end
 outstream(r::LineEditREPL) = r.t
 specialdisplay(r::LineEditREPL) = r.specialdisplay
 specialdisplay(r::AbstractREPL) = nothing
 terminal(r::LineEditREPL) = r.t
+hascolor(r::LineEditREPL) = r.hascolor
 
 LineEditREPL(t::TextTerminal, hascolor::Bool, envcolors::Bool=false) =
     LineEditREPL(t, hascolor,
@@ -452,7 +544,7 @@ function hist_from_file(hp, file, path)
         while !isempty(line)
             push!(lines, chomp(line[2:end]))
             eof(file) && break
-            ch = Char(Base.peek(file))
+            ch = peek(file, Char)
             ch == ' '  && error(munged_history_message(path), countlines)
             ch != '\t' && break
             line = hist_getline(file)
@@ -711,7 +803,7 @@ function eval_with_backend(ast, backend::REPLBackendRef)
     take!(backend.response_channel) # (val, iserr)
 end
 
-function respond(f, repl, main; pass_empty = false)
+function respond(f, repl, main; pass_empty = false, suppress_on_semicolon = true)
     return function do_respond(s, buf, ok)
         if !ok
             return transition(s, :abort)
@@ -726,7 +818,8 @@ function respond(f, repl, main; pass_empty = false)
             catch
                 response = (catch_stack(), true)
             end
-            print_response(repl, response, !ends_with_semicolon(line), Base.have_color)
+            hide_output = suppress_on_semicolon && ends_with_semicolon(line)
+            print_response(repl, response, !hide_output, hascolor(repl))
         end
         prepare_next(repl)
         reset_state(s)
@@ -794,15 +887,10 @@ function setup_interface(
     #
     # This function returns the main interface that describes the REPL
     # functionality, it is called internally by functions that setup a
-    # Terminal-based REPL frontend, but if you want to customize your REPL
-    # or embed the REPL in another interface, you may call this function
-    # directly and append it to your interface.
+    # Terminal-based REPL frontend.
     #
-    # Usage:
-    #
-    # repl_channel,response_channel = Channel(),Channel()
-    # start_repl_backend(repl_channel, response_channel)
-    # setup_interface(REPLDisplay(t),repl_channel,response_channel)
+    # See run_frontend(repl::LineEditREPL, backend::REPLBackendRef)
+    # for usage
     #
     ###
 
@@ -837,7 +925,7 @@ function setup_interface(
         repl = repl,
         complete = replc,
         # When we're done transform the entered line into a call to help("$line")
-        on_done = respond(helpmode, repl, julia_prompt, pass_empty=true))
+        on_done = respond(helpmode, repl, julia_prompt, pass_empty=true, suppress_on_semicolon=false))
 
     # Set up shell mode
     shell_mode = Prompt("shell> ";
@@ -873,7 +961,8 @@ function setup_interface(
             end
             hist_from_file(hp, f, hist_path)
         catch
-            print_response(repl, (catch_stack(),true), true, Base.have_color)
+            # use REPL.hascolor to avoid using the local variable with the same name
+            print_response(repl, (catch_stack(),true), true, REPL.hascolor(repl))
             println(outstream(repl))
             @info "Disabling history file for this session"
             repl.history_file = false
@@ -1053,6 +1142,8 @@ function run_frontend(repl::LineEditREPL, backend::REPLBackendRef)
     repl.backendref = backend
     repl.mistate = LineEdit.init_state(terminal(repl), interface)
     run_interface(terminal(repl), interface, repl.mistate)
+    # Terminate Backend
+    put!(backend.repl_channel, (nothing, -1))
     dopushdisplay && popdisplay(d)
     nothing
 end
@@ -1071,6 +1162,7 @@ StreamREPL(stream::IO) = StreamREPL(stream, Base.text_colors[:green], Base.input
 run_repl(stream::IO) = run_repl(StreamREPL(stream))
 
 outstream(s::StreamREPL) = s.stream
+hascolor(s::StreamREPL) = get(s.stream, :color, false)
 
 answer_color(r::LineEditREPL) = r.envcolors ? Base.answer_color() : r.answer_color
 answer_color(r::StreamREPL) = r.answer_color
@@ -1129,7 +1221,7 @@ function ends_with_semicolon(line::AbstractString)
 end
 
 function run_frontend(repl::StreamREPL, backend::REPLBackendRef)
-    have_color = Base.have_color
+    have_color = hascolor(repl)
     Base.banner(repl.stream)
     d = REPLDisplay(repl)
     dopushdisplay = !in(d,Base.Multimedia.displays)
