@@ -153,11 +153,18 @@ function complete_symbol(sym, ffunc, context_module=Main)::Vector{Completion}
     else
         # Looking for a member of a type
         if t isa DataType && t != Any
-            fields = fieldnames(t)
-            for field in fields
-                s = string(field)
-                if startswith(s, name)
-                    push!(suggestions, FieldCompletion(t, field))
+            # Check for cases like Type{typeof(+)}
+            if t isa DataType && t.name === Base._TYPE_NAME
+                t = typeof(t.parameters[1])
+            end
+            # Only look for fields if this is a concrete type
+            if isconcretetype(t)
+                fields = fieldnames(t)
+                for field in fields
+                    s = string(field)
+                    if startswith(s, name)
+                        push!(suggestions, FieldCompletion(t, field))
+                    end
                 end
             end
         end
@@ -511,7 +518,7 @@ function bslash_completions(string, pos)::Tuple{Bool, Completions}
     return (false, (Completion[], 0:-1, false))
 end
 
-function dict_identifier_key(str,tag)
+function dict_identifier_key(str, tag, context_module = Main)
     if tag === :string
         str_close = str*"\""
     elseif tag === :cmd
@@ -522,7 +529,7 @@ function dict_identifier_key(str,tag)
 
     frange, end_of_identifier = find_start_brace(str_close, c_start='[', c_end=']')
     isempty(frange) && return (nothing, nothing, nothing)
-    obj = Main
+    obj = context_module
     for name in split(str[frange[1]:end_of_identifier], '.')
         Base.isidentifier(name) || return (nothing, nothing, nothing)
         sym = Symbol(name)
@@ -574,7 +581,7 @@ function completions(string, pos, context_module=Main)::Completions
     inc_tag = Base.incomplete_tag(Meta.parse(partial, raise=false, depwarn=false))
 
     # if completing a key in a Dict
-    identifier, partial_key, loc = dict_identifier_key(partial,inc_tag)
+    identifier, partial_key, loc = dict_identifier_key(partial, inc_tag, context_module)
     if identifier !== nothing
         matches = find_dict_matches(identifier, partial_key)
         length(matches)==1 && (lastindex(string) <= pos || string[nextind(string,pos)] != ']') && (matches[1]*=']')
@@ -612,7 +619,9 @@ function completions(string, pos, context_module=Main)::Completions
     inc_tag==:string && return String[], 0:-1, false
     if inc_tag === :other && should_method_complete(partial)
         frange, method_name_end = find_start_brace(partial)
-        ex = Meta.parse(partial[frange] * ")", raise=false, depwarn=false)
+        # strip preceding ! operator
+        s = replace(partial[frange], r"\!+([^=\(]+)" => s"\1")
+        ex = Meta.parse(s * ")", raise=false, depwarn=false)
 
         if isa(ex, Expr)
             if ex.head==:call
@@ -627,6 +636,10 @@ function completions(string, pos, context_module=Main)::Completions
 
     dotpos = something(findprev(isequal('.'), string, pos), 0)
     startpos = nextind(string, something(findprev(in(non_identifier_chars), string, pos), 0))
+    # strip preceding ! operator
+    if (m = match(r"^\!+", string[startpos:pos])) !== nothing
+        startpos += length(m.match)
+    end
 
     ffunc = (mod,x)->true
     suggestions = Completion[]
