@@ -310,7 +310,7 @@ Pick a random element or array of random elements from the set of values specifi
 * a string (considered as a collection of characters), or
 * a type: the set of values to pick from is then equivalent to `typemin(S):typemax(S)` for
   integers (this is not applicable to [`BigInt`](@ref)), to ``[0, 1)`` for floating
-  point numbers and to ``[0, 1)+i[0, 1)]`` for complex floating point numbers;
+  point numbers and to ``[0, 1)+i[0, 1)`` for complex floating point numbers;
 
 `S` defaults to [`Float64`](@ref).
 When only one argument is passed besides the optional `rng` and is a `Tuple`, it is interpreted
