@@ -1,14 +1,18 @@
 # This file is a part of Julia. License is MIT: https://julialang.org/license
 
+import Base.StackTraces: lookup
+
 # Test location information for inlined code (ref issues #1334 #12544)
 module test_inline_bt
 using Test
+import ..lookup
 
 function get_bt_frames(functionname, bt)
     for i = 1:length(bt)
-        lkup = Base.StackTraces.lookup(bt[i])
-        lkup[end].func == functionname && (return lkup)
+        lkup = lookup(bt[i])
+        lkup[end].func == functionname && return lkup
     end
+    return StackTraces.StackFrame[]
 end
 
 # same-file inline
@@ -91,14 +95,14 @@ let
 end
 
 module BackTraceTesting
-
 using Test
+import ..lookup
 
 @inline bt2() = backtrace()
 @inline bt1() = bt2()
 bt() = bt1()
 
-lkup = map(StackTraces.lookup, bt())
+lkup = map(lookup, bt())
 hasbt = hasbt2 = false
 for sfs in lkup
     for sf in sfs
@@ -117,7 +121,7 @@ function btmacro()
     ret = @timed backtrace()
     ret[1]
 end
-lkup = map(StackTraces.lookup, btmacro())
+lkup = map(lookup, btmacro())
 hasme = hasbtmacro = false
 for sfs in lkup
     for sf in sfs
@@ -142,7 +146,7 @@ bt = eval(quote
         catch_backtrace()
     end
 end)
-lkup = map(StackTraces.lookup, bt)
+lkup = map(lookup, bt)
 hastoplevel = false
 for sfs in lkup
     for sf in sfs
@@ -170,7 +174,7 @@ let bt, found = false
     @testset begin
         bt = backtrace()
     end
-    for frame in map(StackTraces.lookup, bt)
+    for frame in map(lookup, bt)
         if frame[1].line == @__LINE__() - 3 && frame[1].file == Symbol(@__FILE__)
             found = true; break
         end
@@ -182,10 +186,75 @@ end
 let bt, found = false
     @info ""
     bt = backtrace()
-    for frame in map(StackTraces.lookup, bt)
+    for frame in map(lookup, bt)
         if frame[1].line == @__LINE__() - 2 && frame[1].file == Symbol(@__FILE__)
             found = true; break
         end
     end
     @test found
 end
+
+# Syntax error locations appear in backtraces
+let trace = try
+        include_string(@__MODULE__,
+            """
+
+            )
+
+            """, "a_filename")
+    catch
+        stacktrace(catch_backtrace())
+    end
+    @test trace[1].func == Symbol("top-level scope")
+    @test trace[1].file == :a_filename
+    @test trace[1].line == 2
+end
+let trace = try
+        include_string(@__MODULE__,
+            """
+
+            incomplete_syntax(
+
+            """, "a_filename")
+    catch
+        stacktrace(catch_backtrace())
+    end
+    @test trace[1].func == Symbol("top-level scope")
+    @test trace[1].file == :a_filename
+    @test trace[1].line == 2
+end
+
+# issue #29695 (see also test for #28442)
+let code = """
+    f29695(c) = g29695(c)
+    g29695(c) = c >= 1000 ? (return backtrace()) : f29695(c + 1)
+    bt = f29695(1)
+    meth_names = [ip.code.def.name for ip in bt
+                  if ip isa Base.InterpreterIP && ip.code isa Core.MethodInstance]
+    num_fs = sum(meth_names .== :f29695)
+    num_gs = sum(meth_names .== :g29695)
+    print(num_fs, ' ', num_gs)
+    """
+
+    @test read(`$(Base.julia_cmd()) --startup-file=no --compile=min -e $code`, String) == "1000 1000"
+end
+
+# Test that modules make it into InterpreterIP for top-level code
+let code = """
+    module A
+    foo() = error("Expected")
+    try
+        foo()
+    catch
+        global bt = catch_backtrace()
+    end
+    end
+
+    foreach(println, A.bt)
+    """
+
+    bt_str = read(`$(Base.julia_cmd()) --startup-file=no --compile=min -e $code`, String)
+    @test occursin("InterpreterIP in MethodInstance for foo", bt_str)
+    @test occursin("InterpreterIP in top-level CodeInfo for Main.A", bt_str)
+end
+
