@@ -1751,3 +1751,14 @@ len,point,neg = Grisu.grisu(1.0, Grisu.FIXED, 0, buffer)
 @test 1 >= len-1
 @test "1" == unsafe_string(pointer(buffer))
 @test !neg
+
+# issue #29885
+@sync let p = Pipe(), q = Pipe()
+    Base.link_pipe!(p, reader_supports_async=true, writer_supports_async=true)
+    Base.link_pipe!(q, reader_supports_async=true, writer_supports_async=true)
+    @async write(p, zeros(UInt8, 2^18))
+    @async (print(p, 12.345); close(p.in))
+    @async print(q, 9.8)
+    read(p, 2^18)
+    @test read(p, String) == "12.345"
+end
